/*
 * Copyright 2021 Intrinsic ID B.V. All rights reserved.
 *
 * Usage of this software is permitted under a valid written license agreement
 * between you and Intrinsic ID B.V.
 */

#include <string.h>

#include "bk_demo_app.h"
#include "iidbroadkey.h"
#include "iid_target.h"

#include "endian.h"

#define SYMMETRIC_OUT_KEY_SIZE_BYTES	(16)

#define PRINT_WIDTH						16

#define PRE_HIS_ALIGN
#define POST_HIS_ALIGN					__attribute__((aligned(4)))


//#define GW1NS_9C_SEC


/**************************** STATIC DECLARATIONS *************************/
static uint8_t demo_version(void);
static uint8_t demo_init(uint8_t * const sram_puf);
static uint8_t demo_start(uint8_t * const sram_ac);
static uint8_t demo_get_random_bytes(void);
static uint8_t demo_get_symmetric_key(void);
static uint8_t demo_get_private_key(void);

static uint8_t demo_wrap_and_unwrap(void);

static uint8_t demo_derive_public_key_from_private_key(void);
static uint8_t demo_create_private_and_public_key_codes(bk_ecc_private_key_code_t * const ecc_private_key_code,
                                                        bk_ecc_public_key_code_t  * const ecc_public_key_code);
static uint8_t demo_import_and_export_public_key(void);
static uint8_t demo_ecdsa_sign_and_verify(const bk_ecc_private_key_code_t * const ecc_private_key_code,
                                          const bk_ecc_public_key_code_t  * const ecc_public_key_code,
                                          const uint8_t                   * const message,
                                          const uint32_t                          message_length);
static uint8_t demo_ecdh_shared_secret(const bk_ecc_private_key_code_t * const ecc_private_key_code,
                                       const bk_ecc_public_key_code_t  * const ecc_public_key_code);

#ifdef GW1NS_9C_SEC
static uint8_t demo_generate_and_process_cryptogram(const bk_ecc_private_key_code_t * const sender_ecc_private_key_code,
                                                    const bk_ecc_public_key_code_t  * const sender_ecc_public_key_code);
#endif

static void print_fpga_prologue(char * const func_name);
static void print_fpga_error(char * const func_name);
static void print_bk_error(char * const func_name, uint32_t err_code);


/************************** PUBLIC FUNCTIONS *****************************/
void bk_demo_app(uint8_t * const sram_puf, uint8_t * const sram_ac)
{
	PRE_HIS_ALIGN bk_ecc_private_key_code_t ecc_private_key_code POST_HIS_ALIGN;
	PRE_HIS_ALIGN bk_ecc_public_key_code_t ecc_public_key_code POST_HIS_ALIGN;
	PRE_HIS_ALIGN uint8_t message[32] POST_HIS_ALIGN = {0xab, 0xcd, 0xef, 0x10, 0x23, 0x45, 0x67, 0x89,
	                                                    0xab, 0xcd, 0xef, 0x10, 0x23, 0x45, 0x67, 0x89,
	                                                    0xab, 0xcd, 0xef, 0x10, 0x23, 0x45, 0x67, 0x89,
	                                                    0xab, 0xcd, 0xef, 0x10, 0x23, 0x45, 0x67, 0x89};
	iid_return_t retval = 0;

	PRINTF("\r\n**** Welcome to GowinSemiconductor Secure FPGA (GW1NS_9C) Demo ****\r\n");
	print_fpga_prologue("Get Version");
	/***********************
	* BroadKey Version     *
	***********************/
	retval = demo_version();
	if (retval)
	{
		print_fpga_error("Get Version");
		goto EXIT;
	}

	print_fpga_prologue("Initialize");
	/***********************
	 * Initialize BroadKey *
	 ***********************/
	retval = demo_init(sram_puf);
	if (retval)
	{
		print_fpga_error("Initialize");
		goto EXIT;
	}

	print_fpga_prologue("Start");
	/******************************
	 *  BroadKey Start  *
	 ******************************/
	retval = demo_start(sram_ac);
	if (retval)
	{
		print_fpga_error("Start");
		goto EXIT;
	}

	print_fpga_prologue("Generate Random Bytes");
	/********************************
	 *  BroadKey  Get Random Bytes  *
	 ********************************/
	retval = demo_get_random_bytes();
	if(retval)
	{
		print_fpga_error("Generate Random Bytes");
		goto EXIT;
	}

	print_fpga_prologue("Get Symmetric Key");
	/*********************************
	 *  BroadKey  Get Symmetric Key  *
	 *********************************/
	retval = demo_get_symmetric_key();
	if(retval)
	{
		print_fpga_error("Get Symmetric Key");
		goto EXIT;
	}

	print_fpga_prologue("Get ECC Private Key");
	/***********************************
	 *  BroadKey  Get ECC Private Key  *
	 ***********************************/
	retval = demo_get_private_key();
	if(retval)
	{
		print_fpga_error("Get ECC Private Key");
		goto EXIT;
	}

	print_fpga_prologue("Wrap and Unwrap the (user) key");
	retval = demo_wrap_and_unwrap();
	if(retval)
	{
		print_fpga_error("Wrap and Unwrap the (user) key");
		goto EXIT;
	}

	/****************************************************
	 *  BroadKey GET PRIVATE KEY AND DERIVE PUBLIC KEY  *
	 ****************************************************/
	print_fpga_prologue("Get Private Key and Derive Public Key");
	retval = demo_derive_public_key_from_private_key();
	if(retval)
	{
		print_fpga_error("Get Private Key and Derive Public Key");
		goto EXIT;
	}

	/*************************************************
	 *  BroadKey CREATE PRIVATE AND PUBLIC KEY CODE  *
	 *************************************************/
	print_fpga_prologue("Create Private and Public Key Code");
	retval = demo_create_private_and_public_key_codes(&ecc_private_key_code, &ecc_public_key_code);
	if(retval)
	{
		print_fpga_error("Create Private and Public Key Code");
		goto EXIT;
	}

	/*******************************************
	 *  BroadKey IMPORT AND EXPORT PUBLIC KEY  *
	 *******************************************/
	print_fpga_prologue("Import and Export Public Key");
	retval = demo_import_and_export_public_key();
	if(retval)
	{
		print_fpga_error("Import and Export Public Key");
		goto EXIT;
	}

	/****************************************
	 *  BroadKey ECDSA SIGN AND VERIFY KEY  *
	 ****************************************/
	print_fpga_prologue("ECDSA Sign and Verify");
	retval = demo_ecdsa_sign_and_verify(&ecc_private_key_code, &ecc_public_key_code, message, sizeof(message));
	if(retval)
	{
		print_fpga_error("ECDSA Sign and Verify");
		goto EXIT;
	}

	/*********************************
	 *  BroadKey ECDH SHARED SECRET  *
	 *********************************/
	print_fpga_prologue("Get Shared Secret");
	retval = demo_ecdh_shared_secret(&ecc_private_key_code, &ecc_public_key_code);
	if(retval)
	{
		print_fpga_error("Get Shared Secret");
		goto EXIT;
	}

#ifdef GW1NS_9C_SEC
	/*************************************
	 *  GENERATE AND PROCESS CRYPTOGRAM  *
	 *************************************/
	print_fpga_prologue("Generate and Process Cryptogram");
	retval = demo_generate_and_process_cryptogram(&ecc_private_key_code, &ecc_public_key_code);
	if(retval)
	{
		print_fpga_error("Generate and Process Cryptogram");
		goto EXIT;
	}
#endif /* GW1NS_9C_SEC */

EXIT:
	if(retval)
	{
		print_fpga_error("BroadKey Demo");
	}
	else
	{
		PRINTF("\r\n--- %s! ---\r\n", "Secure FPGA BroadKey Demo Successfully");
		PRINTF("\r\n--- %s! ---\r\n", "End of the Demo Application");
	}

	return;
};


/*********************
*  BroadKey Version  *
**********************/
static uint8_t demo_version(void)
{
	uint8_t major_version;
	uint8_t minor_version;
	uint8_t product_id;
	uint8_t patch;
	uint8_t build_number;
	iid_return_t retval;

	retval = bk_get_product_info(&product_id,
	                             &major_version,
	                             &minor_version,
	                             &patch,
	                             &build_number);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_get_product_info", retval);
		return 1;
	}

	PRINTF("product_id     : %c\r\n", product_id);
	PRINTF("major_version  : %d\r\n", major_version);
	PRINTF("nminor_version : %d\r\n", minor_version);
	PRINTF("patch          : %d\r\n", patch);
	PRINTF("build_number   : %d\r\n", build_number);

	return 0;
}


/*************************
 *  Initialize BroadKey  *
 *************************/
static uint8_t demo_init(uint8_t * const sram_puf)
{
	iid_return_t retval;

	PRINTF("%s ...", "bk_init");

	retval = bk_init(sram_puf, BK_SRAM_PUF_SIZE_BYTES);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_init", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	return 0;
}


/****************************
 * BroadKey Start *
 ****************************/
static uint8_t demo_start(uint8_t * const sram_ac)
{
	PRE_HIS_ALIGN uint8_t activation_code[BK_AC_SIZE_BYTES] POST_HIS_ALIGN;
	unsigned int i;
	iid_return_t retval;
	uint8_t input;

	PRINTF("Secure FPGA Demo run:\r\n");
	PRINTF("UART input %d for demo start(the ac code is %s endian).\r\n", 1, "big");
	PRINTF("UART input %d for demo start(the ac code is %s endian).\r\n", 2, "little");

	do
	{
		PRINTF("Input : ");

		input = target_input_char();

		PRINTF("%c\r\n", input);
	} while (('1' != input) && ('2' != input));

	PRINTF("AC Addr : %08X\r\n", (uint32_t)sram_ac);

	memcpy(activation_code, sram_ac, BK_AC_SIZE_BYTES);

	PRINTF("The activation code is ");
	if ('1' == input)
	{
		PRINTF("%s endian\r\n", "big");
		endian_big_to_little_array((uint32_t *)activation_code, BK_AC_SIZE_BYTES / 4);
	}
	else
	{
		PRINTF("%s endian\r\n", "little");
	}

	PRINTF("00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n");

	print_hex_string(activation_code, BK_AC_SIZE_BYTES);

	PRINTF("00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\r\n");

	PRINTF("%s ...", "bk_start");

	retval = bk_start(activation_code);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_start", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	return 0;
}


/********************************
 *  BroadKey  Get Random Bytes  *
 ********************************/
static uint8_t demo_get_random_bytes(void)
{
	uint16_t num_bytes = 16;
	uint8_t  data_buffer[16];
	unsigned i;
	iid_return_t retval;

	PRINTF("%s ...", "bk_generate_random");

	retval = bk_generate_random(num_bytes, data_buffer);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_generate_random", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The Random generated bytes are :\r\n");

	print_hex_string(data_buffer, num_bytes);

	return 0;
}


/************************
 *  BroadKey  Get Key   *
 ************************/
static uint8_t demo_get_symmetric_key(void)
{
	PRE_HIS_ALIGN uint8_t key[SYMMETRIC_OUT_KEY_SIZE_BYTES] POST_HIS_ALIGN;
	unsigned int i;
	uint8_t index = 0;
	iid_return_t retval;

	PRINTF("%s ...", "bk_get_key");

	retval = bk_get_key(BK_SYM_KEY_TYPE_128, index, key);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_get_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The Key is :\r\n");

	print_hex_string(key, SYMMETRIC_OUT_KEY_SIZE_BYTES);

	return 0;
}


/********************************
 *  BroadKey Get ECC PRIVATE Key    *
 ********************************/
static uint8_t demo_get_private_key(void)
{
	bk_ecc_curve_t curve = BK_ECC_CURVE_NIST_P224;
	/* Size of the private key depends on the curve, see datasheet */
	PRE_HIS_ALIGN uint8_t private_key[BK_ECC_CURVE_SECP224R1_PRIVATE_KEY_BYTES] POST_HIS_ALIGN;
	iid_return_t retval;
	unsigned int i;

	PRINTF("%s ...", "bk_get_private_key");

	retval = bk_get_private_key(curve,
	                            NULL, /* No Usage Context */
	                            0,
	                            BK_ECC_KEY_SOURCE_PUF_DERIVED,
	                            private_key);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_get_private_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The generated ECC Private Key is:\r\n");

	print_hex_string(private_key, sizeof(private_key));

	return 0;
}


/****************************
 * BroadKey Wrap and UnWrap *
 ****************************/
static uint8_t demo_wrap_and_unwrap(void)
{
	PRE_HIS_ALIGN uint8_t user_key[16] POST_HIS_ALIGN = {0x66,0xf4,0x77,0xce,0xbe,0x0e,0x51,0x71,0x6a,0x73,0x3c,0x92,0xe2,0x8f,0xcb,0x0c};
	PRE_HIS_ALIGN uint8_t ret_user_key[sizeof(user_key)] POST_HIS_ALIGN = {0};
	PRE_HIS_ALIGN uint8_t key_code[sizeof(user_key) + BK_USER_KEY_CODE_NONKEY_BYTES] POST_HIS_ALIGN;
	uint32_t i;
	uint8_t index = 0;
	uint8_t ret_index;
	uint16_t ret_uk_length;
	iid_return_t retval;

	PRINTF("The User Key is:\r\n");

	print_hex_string(user_key, sizeof(user_key));

	/***********************
	 *  BroadKey  WRAP    *
	 ***********************/
	PRINTF("%s ...", "bk_wrap");

	retval = bk_wrap(index, user_key, sizeof(user_key), key_code);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_wrap", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The Key Code is:\r\n");

	print_hex_string(key_code, sizeof(key_code));

	/***********************
	 *  BroadKey  UNWRAP    *
	 ***********************/
	PRINTF("%s ...", "bk_unwrap");

	retval = bk_unwrap(key_code, ret_user_key, &ret_uk_length, &ret_index);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_unwrap", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The Ret User Key is:\r\n");

	print_hex_string(ret_user_key, sizeof(ret_user_key));

	PRINTF("\r\nReturn index: %d", ret_index);

	PRINTF("\r\nReturn user key length: %d", ret_uk_length);

	return 0;
}


/****************************************************
 *  BroadKey GET PRIVATE KEY AND DERIVE PUBLIC KEY  *
 ****************************************************/
static uint8_t demo_derive_public_key_from_private_key(void)
{
	bk_ecc_curve_t curve = BK_ECC_CURVE_NIST_P192;
	/* Size of the private key depends on the curve, see datasheet */
	PRE_HIS_ALIGN uint8_t private_key[BK_ECC_CURVE_SECP192R1_PRIVATE_KEY_BYTES] POST_HIS_ALIGN;
	/* Size of the public key depends on the curve, see datasheet */
	PRE_HIS_ALIGN uint8_t public_key[BK_ECC_CURVE_SECP192R1_PUBLIC_KEY_BYTES] POST_HIS_ALIGN;
	uint8_t usage_context[] = {'D', 'E', 'M', 'O', 'K', 'E', 'Y'};
	uint32_t i;
	iid_return_t retval;

	/******************************
	 *  BroadKey GET PRIVATE KEY  *
	 ******************************/
	PRINTF("%s ...", "bk_get_private_key");

	retval = bk_get_private_key(
	                curve,
	                usage_context,
	                sizeof(usage_context),
	                BK_ECC_KEY_SOURCE_RANDOM,
	                private_key);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_get_private_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The generated Private Key is:\r\n");

	print_hex_string(private_key, sizeof(private_key));

	/******************************
	 *  BroadKey DERIVE PUBLIC KEY  *
	 ******************************/
	PRINTF("%s ...", "bk_derive_public_key");

	retval = bk_derive_public_key(false, curve, private_key, public_key);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_derive_public_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The derived Public Key is:\r\n");

	print_hex_string(public_key, sizeof(public_key));

	return 0;
}


static uint8_t demo_create_private_and_public_key_codes(bk_ecc_private_key_code_t * const ecc_private_key_code,
                                                        bk_ecc_public_key_code_t  * const ecc_public_key_code)
{
	bk_ecc_curve_t curve;
	bk_ecc_key_purpose_t purpose_flags;
	bk_ecc_key_source_t key_source;
	uint32_t i;
	iid_return_t retval;

	/****************************************
	 *  BroadKey CREATE PRIVATE KEY CODE    *
	 ****************************************/
	curve = BK_ECC_CURVE_NIST_P256;
	purpose_flags = BK_ECC_KEY_PURPOSE_ECDH_AND_ECDSA;
	key_source = BK_ECC_KEY_SOURCE_PUF_DERIVED;

	PRINTF("%s ...", "bk_create_private_key");

	retval = bk_create_private_key(curve,
	                                purpose_flags,
	                                NULL,
	                                0,
	                                key_source,
	                                NULL,
	                                ecc_private_key_code);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_create_private_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The Private Key Code is:\r\n");

	print_hex_string((uint8_t *)ecc_private_key_code, BK_ECC_PRIVATE_KEY_CODE_SIZE_BYTES);

	/********************************************************
	 *  BroadKey CREATE PUBLIC KEY FROM PRIVATE KEY CODE    *
	 ********************************************************/
	PRINTF("%s ...", "bk_compute_public_from_private_key");

	retval = bk_compute_public_from_private_key(ecc_private_key_code, ecc_public_key_code);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_compute_pubic_key_from_private_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The ECC Public Key Code is:\r\n");

	print_hex_string((uint8_t *)ecc_public_key_code, BK_ECC_PUBLIC_KEY_CODE_SIZE_BYTES);

	return 0;
}

static uint8_t demo_import_and_export_public_key(void)
{
	bk_ecc_curve_t curve = BK_ECC_CURVE_NIST_P256;
	/* Size of the public key depends on the curve. 65 is according to NIST P256 curve */
	PRE_HIS_ALIGN uint8_t ecc_x9_62_public_key[BK_ECC_CURVE_SECP256R1_PUBLIC_KEY_BYTES] POST_HIS_ALIGN =
	{
		0x04, 0xf3, 0x13, 0x94, 0x7a, 0xdb, 0x02, 0x6b, 0xc1, 0x1f, 0x3a, 0x50, 0xc4, 0x04, 0x07, 0xd1,
		0x12, 0x12, 0x6a, 0xb2, 0x90, 0xe8, 0xc8, 0x48, 0x7a, 0x4b, 0x3c, 0xf6, 0x8a, 0x7e, 0x3b, 0x4e,
		0xfe, 0x67, 0xe7, 0x69, 0xbd, 0x74, 0xb6, 0x32, 0x3c, 0x26, 0xea, 0x66, 0x43, 0x81, 0xe6, 0x74,
		0xd2, 0xaa, 0xdb, 0x1c, 0xa6, 0x44, 0x03, 0xfc, 0x7a, 0x72, 0x90, 0x28, 0x20, 0xbf, 0x1d, 0x3d,
		0x89
	};
	PRE_HIS_ALIGN uint8_t temp_ecc_x9_62_public_key[BK_ECC_CURVE_SECP256R1_PUBLIC_KEY_BYTES] POST_HIS_ALIGN = {0};
	PRE_HIS_ALIGN bk_ecc_public_key_code_t ecc_x9_62_public_key_code POST_HIS_ALIGN;
	uint32_t i;
	bk_ecc_key_purpose_t purpose_flags;
	iid_return_t retval;

	PRINTF("The User ECC Public Key, to be imported, is:\r\n");

	print_hex_string(ecc_x9_62_public_key, sizeof(ecc_x9_62_public_key));

	/********************************
	 *  BroadKey IMPORT PUBLIC KEY  *
	 ********************************/
	purpose_flags = BK_ECC_KEY_PURPOSE_ECDH_AND_ECDSA;

	PRINTF("%s ...", "bk_import_public_key");

	retval = bk_import_public_key(curve,
	                              purpose_flags,
	                              ecc_x9_62_public_key,
	                              &ecc_x9_62_public_key_code);
	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_import_public_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The ECC Public Key Code is:\r\n");

	print_hex_string((uint8_t *)ecc_x9_62_public_key_code, BK_ECC_PUBLIC_KEY_CODE_SIZE_BYTES);

	/********************************
	 *  BroadKey EXPORT PUBLIC KEY  *
	 ********************************/
	PRINTF("%s ...", "bk_export_public_key");

	retval = bk_export_public_key(0,
	                              &ecc_x9_62_public_key_code,
	                              temp_ecc_x9_62_public_key,
	                              &curve,
	                              &purpose_flags);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_export_public_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The ECC Public Key is:\r\n");

	print_hex_string(temp_ecc_x9_62_public_key, sizeof(temp_ecc_x9_62_public_key));

	return 0;
}


static uint8_t demo_ecdsa_sign_and_verify(const bk_ecc_private_key_code_t * const ecc_private_key_code,
                                          const bk_ecc_public_key_code_t  * const ecc_public_key_code,
                                          const uint8_t                   * const message,
                                          const uint32_t                          message_length)
{
	/* The key codes have previously been generated with BK_ECC_CURVE_NIST_P256 */
	PRE_HIS_ALIGN uint8_t signature[BK_ECC_CURVE_SECP256R1_SIGNATURE_BYTES] POST_HIS_ALIGN;
	uint16_t signature_length;
	uint8_t message_is_hashed = 0; /* message is not hashed */
	uint32_t i;
	iid_return_t retval;

	/*****************************
	 *  BroadKey ECDSA SIGN KEY  *
	 *****************************/
	signature_length = sizeof(signature);

	PRINTF("%s ...", "bk_ecdsa_sign_key");

	retval = bk_ecdsa_sign(ecc_private_key_code,
	                        0, /* not a deterministic signature */
	                        message,
	                        message_length,
	                        message_is_hashed,
	                        signature,
	                        &signature_length);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_ecdsa_sign_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The ECDSA signature is:\r\n");

	print_hex_string(signature, signature_length);

	/*******************************
	 *  BroadKey ECDSA VERIFY KEY  *
	 *******************************/
	PRINTF("%s ...", "bk_ecdsa_verify");

	retval =  bk_ecdsa_verify(ecc_public_key_code,
	                        message,
	                        message_length,
	                        message_is_hashed,
	                        signature,
	                        signature_length);

	if (IID_SUCCESS != retval)
	{
		if (IID_INVALID_SIGNATURE == retval)
		{
			PRINTF("Invalid signature to the given message\r\n");
		}
		else
		{
			print_bk_error("bk_ecdsa_verify", retval);
		}

		return 1;
	}

	PRINTF("Signature Verified! \r\ndone!\r\n");

	return 0;
}

static uint8_t demo_ecdh_shared_secret(const bk_ecc_private_key_code_t * const ecc_private_key_code,
                                       const bk_ecc_public_key_code_t  * const ecc_public_key_code)
{
	/* The key codes have previously been generated with BK_ECC_CURVE_NIST_P256 */
	PRE_HIS_ALIGN uint8_t shared_secret[BK_ECC_CURVE_SECP256R1_SHARED_SECRET_BYTES] POST_HIS_ALIGN;
	uint32_t i;
	iid_return_t retval;

	PRINTF("%s ...", "bk_ecdh_shared_secret");

	retval = bk_ecdh_shared_secret(ecc_private_key_code,
	                               ecc_public_key_code,
	                               shared_secret);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_ecdh_shared_secret", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The ECDH shared secret is:\r\n");

	print_hex_string(shared_secret, sizeof(shared_secret));

	return 0;
}


#ifdef GW1NS_9C_SEC
static uint8_t demo_generate_and_process_cryptogram(const bk_ecc_private_key_code_t * const sender_ecc_private_key_code,
                                                    const bk_ecc_public_key_code_t  * const sender_ecc_public_key_code)
{
	PRE_HIS_ALIGN uint8_t rec_private_key[32] POST_HIS_ALIGN = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	                                                            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	                                                            0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	                                                            0x00,0x00,0x00,0x2B,0x12,0x0F,0x67,0x21};
	PRE_HIS_ALIGN bk_ecc_private_key_code_t rec_ecc_private_key_code POST_HIS_ALIGN;
	PRE_HIS_ALIGN bk_ecc_public_key_code_t rec_ecc_public_key_code POST_HIS_ALIGN;
	PRE_HIS_ALIGN uint8_t plaintext[32] POST_HIS_ALIGN = {'I', 'N', 'T', 'R', 'I', 'N', 'S', 'I',
	                                                      'C', 0xcd, 0xef, 0x10, 0x23, 0x45, 0x67, 0x89,
	                                                      0xab, 0xcd, 0xef, 0x10, 0x23, 0x45, 0x67, 0x89,
	                                                      0xab, 0xcd, 0xef, 0x10, 0x23, 0x45, 0x67, 0x89};
	PRE_HIS_ALIGN uint8_t rec_plaintext[sizeof(plaintext)] POST_HIS_ALIGN;
	PRE_HIS_ALIGN uint8_t cryptogram[BK_ECC_CRYPTOGRAM_HEADER_SIZE_BYTES + 64 + sizeof(plaintext)] POST_HIS_ALIGN;
	PRE_HIS_ALIGN uint32_t cryptogram_length POST_HIS_ALIGN = sizeof(cryptogram);
	PRE_HIS_ALIGN uint32_t rec_plaintext_length POST_HIS_ALIGN = sizeof(rec_plaintext);
	PRE_HIS_ALIGN uint8_t derived_public_key_from_cryptogram[65] POST_HIS_ALIGN; /* size depends on the curve, here it is maximum to support all */
	uint8_t counter64_sender[8] = { 0 }; /* counter is set to all zeros */
	uint8_t counter64_receiver[8] = { 0 };
	bk_ecc_cryptogram_type_t cryptogram_type;
	bk_ecc_curve_t curve;
	bk_ecc_key_purpose_t purpose_flags;
	bk_ecc_key_source_t key_source;
	uint32_t i;
	iid_return_t retval;

	/*************************************************************************
	 *  BroadKey CREATE PRIVATE KEY CODE FROM USER PROVIDED KEY FOR RECEIVER *
	 *************************************************************************/
	curve = BK_ECC_CURVE_NIST_P256;
	purpose_flags = BK_ECC_KEY_PURPOSE_ECDH_AND_ECDSA;
	key_source = BK_ECC_KEY_SOURCE_USER_PROVIDED;

	PRINTF("%s ...", "bk_create_private_key");

	retval = bk_create_private_key(curve,
	                               purpose_flags,
	                               NULL, /* Not using usage context */
	                               0, /* Not using usage length */
	                               key_source,
	                               rec_private_key,
	                               &rec_ecc_private_key_code);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_create_private_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The RECEIVER Private Key Code is:\r\n");

	print_hex_string(rec_ecc_private_key_code, BK_ECC_PRIVATE_KEY_CODE_SIZE_BYTES);

	/************************************************************
	 *  BroadKey CREATE PUBLIC KEY CODE FOR RECEIVER            *
	 ************************************************************/
	PRINTF("%s ...", "bk_compute_public_from_private_key");

	retval = bk_compute_public_from_private_key(&rec_ecc_private_key_code, &rec_ecc_public_key_code);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_compute_pubic_key_from_private_key", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The ECC  RECEIVER Public Key Code is:\r\n");

	print_hex_string(rec_ecc_public_key_code, BK_ECC_PUBLIC_KEY_CODE_SIZE_BYTES);

	/*********************************
	 * GENERATE CRYPTOGRAM BY SENDER *
	 *********************************/
	PRINTF("%s ...", "bk_generate_cryptogram");

	retval = bk_generate_cryptogram(&rec_ecc_public_key_code,
	                                sender_ecc_private_key_code,
	                                BK_ECC_CRYPTOGRAM_TYPE_ECDH_STATIC,
	                                counter64_sender,
	                                plaintext,
	                                sizeof(plaintext),
	                                cryptogram,
	                                &cryptogram_length);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_generate_cryptogram", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("GENERATED CRYPTOGRAM is:\r\n");

	print_hex_string(cryptogram, cryptogram_length);

	/******************************************
	 * GET SENDER PUBLIC KEY FROM CRYPTOGRAM  *
	 ******************************************/
	PRINTF("%s ...", "bk_get_public_key_from_cryptogram");

	retval = bk_get_public_key_from_cryptogram(0,
	                                           curve,
	                                           cryptogram,
	                                           cryptogram_length,
	                                           derived_public_key_from_cryptogram);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_get_public_key_from_cryptogram", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("The PUBLIC KEY from CRYPTOGRAM is:\r\n");

	print_hex_string(derived_public_key_from_cryptogram, sizeof(derived_public_key_from_cryptogram));

	/**********************************
	 * PROCESS CRYPTOGRAM BY RECEIVER *
	 **********************************/
	PRINTF("%s ...", "bk_process_cryptogram");

	retval = bk_process_cryptogram(&rec_ecc_private_key_code,
	                                sender_ecc_public_key_code,
	                                &cryptogram_type,
	                                counter64_receiver,
	                                cryptogram,
	                                cryptogram_length,
	                                rec_plaintext,
	                                &rec_plaintext_length);

	if (IID_SUCCESS != retval)
	{
		print_bk_error("bk_process_cryptogram", retval);
		return 1;
	}

	PRINTF("done!\r\n");

	PRINTF("RECIEVED PLAINTEXT is:\r\n");

	print_hex_string(rec_plaintext, sizeof(rec_plaintext));
	PRINTF("\r\n");

	return 0;
}
#endif /* GW1NS_9C_SEC */


static void print_fpga_prologue(char * const func_name)
{
	PRINTF("\r\n---%s---\r\n", func_name);
}


static void print_fpga_error(char * const func_name)
{
	PRINTF("\r\n--- Secure FPGA %s Failed. ---\r\n", func_name);
}


static void print_bk_error(char * const func_name, uint32_t err_code)
{
	PRINTF("Error: Function \"%s\" failed with return value %08X\r\n", func_name, err_code);
}
