/*
 * Copyright 2021 Intrinsic ID B.V.  All rights reserved.
 *
 * Usage of this software is permitted under a valid written license agreement
 * between you and Intrinsic ID B.V.
 */

/*!
 * \file    iid_target.h
 * \brief   Hardware abstraction layer definition.
 */

#ifndef _IID_TARGET_H_
#define _IID_TARGET_H_

#include <stdbool.h>
#include <stdint.h>

/*******************************************************************************
 * Target specific includes
 ******************************************************************************/
/* TODO: insert target specific include files here. */
#include "GOWIN_M1.h"


/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*
 *  TODO: If a target specific printf function exists, uncomment the next line
 *  and add the target specific function name to the definition.
 */
/* #define PRINTF							... */


#ifndef PRINTF
/*!
 * \brief   Define containing the name of the designated printf function.
 *
 * \details This define must contain the printf function name as used by the target
 *          In case no appropriate target supplied function is found, the function
 *          #target_printf can be used as this contains the required printf functionality.
 *
 *
 */
#define PRINTF target_printf
#endif


/*
 * TODO: Replace XXXX with the size or an existing define containing the smallest number
 * of bytes that can be written to NVM
 */
/*!
 * \brief   The smallest amount of data that can be written to nvm
 *
 * \details This define contains the smallest amount of data that can be written to nvm.
 *          The define is used to calculate the
 */
#define IID_TARGET_NVM_WRITE_BYTES			(XXXX)


/* TODO: Replace XXXX with the largest alignment value of either the NVM or SRAM */
/*!
 * \brief   The smallest nvm/ram data alignment
 *
 * \details This define contains the largest alignment value of either the nvm or the SRAM alignment.
 *          The define is used to ensure that the nvm read, write and compare functions can be executed.
 */
#define IID_TARGET_NVM_DATA_ALIGNMENT		(XXXX)

/* TODO: Add compiler directive to align data if the directive precedes the data declaration */
/*!
 * \brief   The data alignment macro used by BK variables
 *
 * \details This macro contains the compiler directive to align data on a 32 bit address.
 *          The directive is only defined if it precedes a data definition, e.g.
 *          __declspec(align(4)) for the msvc compiler. If a compiler directive exists that
 *          is located after a data declaration, define the #POST_ALIGN macro and define this
 *          macro as empty.
 */
#define PRE_ALIGN

/* TODO: Add compiler directive to align data if the directive follows the data declaration */
/*!
 * \brief   The data alignment macro used by BK variables
 *
 * \details This macro contains the compiler directive to align data on a 32 bit address.
 *          The directive is only defined if it follows a data definition, e.g.
 *          __attribute__((aligned(4))) for the gcc compiler. If a compiler directive exists that
 *          is located after a data declaration, define the #POST_ALIGN macro and define this
 *          macro as empty.
 */
#define POST_ALIGN


/*******************************************************************************
 * Types
 ******************************************************************************/
/*!
 * \brief   Available LED's
 *
 * \details This enumeration defines the available LEDs on the target. These definitions
 *          are required even though they do not exist on the target board.
 *
 * \note    The available LEDs may be mapped onto these definitions at the discretion
 *          of the developer.
 */
typedef enum {
	led_red = 0,		/*!< LED indicating BK-Demo is in enrolled state    */
	led_green,			/*!< LED indicating BK-Demo is in un-enrolled state */
	led_blue			/*!< LED indicating BK-Demo is in the started state */
} e_target_hw_led;


/*******************************************************************************
 * Variables
 ******************************************************************************/


/*******************************************************************************
 * API
 ******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif

/*!
 * \brief   Initialize the target board.
 *
 * \details This function initializes the clocks, peripherals and other hardware
 *          required to run this demo.
 *          - main clock
 *          - peripheral clocks
 *          - GPIO/MUX
 *          - u(s)art
 *          - LEDs
 *          - flash driver
 *
 * \retval  None, any failure in the target initialization should result in an exception.
 */
extern void target_init(void);

/*!
 * \brief     Control a LED on the target board.
 *
 * \details   This function controls a LED on the target board. The LED can be switched On or Off.
 *
 * \param[in] led The LED to switch On of Off
 * \param[in] on  When true switches the LED On, Off otherwise
 *
 * \retval    None, any error should result in an exception.
 */
extern void target_led_control(e_target_hw_led led, bool on);

/*!
 * \brief   Retrieve an upper case character from the uart.
 *
 * \details This function retrieves an upper case character from the designated uart.
 *
 * \return  An upper case ASCII character.
 */
extern char target_get_upper_case_char(void);

/*!
 * \brief     Receive a character from the uart.
 *
 * \details   This function receives a character from the designated uart.
 *
 * \retval    The received character.
 */
extern char target_input_char(void);

/*!
 * \brief     Send a character over the uart.
 *
 * \details   This function transmits a character using the designated uart.
 *
 * \param[in] value The character to transmit
 *
 * \retval    None, any error should result in an exception.
 */
extern void target_output_char(char value);

/*!
 * \brief   Erase the non volatile storage.
 *
 * \details This function erases the non volatile storage on the target. This will
 *          set the target to the un-enrolled BK-Demo state.
 *
 * \note    If the non volatile storage is flash, wiping the BK-Demo designated flash region
 *          is required before data can be written to it.
 *
 * \note    It is the users discretion to either erase pages or a sector of flash.
 *
 * \retval  None, any error should result in an exception.
 */
extern void target_nvm_clear(void);

/*!
 * \brief     Write to the non volatile storage.
 *
 * \details   This function writes the BK-Demo persistent data to non volatile storage.
 *
 * \param[in] src_addr  Source address of data that needs to be stored in nvm.
 * \param[in] num_bytes Number of bytes to be written to nvm.
 *
 * \note      If the non volatile storage is flash, wiping the flash sector is required before
 *            data can be written to it.
 *
 * \note      In case of flash, the implementation depends on the flash geometry, i.e. page size,
 *            data alignment etc.
 *
 * \return    None, any error should result in an exception.
 */
extern void target_nvm_write(const uint32_t * src_addr, uint32_t num_bytes);

/*!
 * \brief      Read from non volatile storage.
 *
 * \details    This function reads the BK-Demo persistent data from volatile storage.

 * \param[out] dest_addr Destination address of data that needs to be read from nvm.
 * \param[in]  num_bytes Number of bytes to be read from nvm.
 *
 * \note       If the non volatile storage is flash, direct reading may be a faster option. However,
 *             since this is an abstraction layer the read function is added from completeness.
 *
 * \return     None, any error should result in an exception.
 */
extern void target_nvm_read(uint32_t * dest_addr, uint32_t num_bytes);

/*!
 * \brief   Reset the target processor.
 *
 * \details This function resets the target processor. For arm processors, a call to the function NVIC_SystemReset
 *          is typically used.
 */
extern void target_nvic_reset(void);

/*!
 * \brief     Implementation of limited std C printf functionality.
 *
 * \details   This function implements a limited version of printf functionality as defined by the standard C definition.
 *            This version supports %c, %s, %d, %x and %X format specifiers and the width specifier with optional
 *            zero padding.
 *
 * \param[in] format C string that contains the text to be written to the communication channel. It can optionally
 *                   contain embedded format specifiers that are replaced by the values specified in the subsequent
 *                   additional arguments and formatted as requested.
 * \param[in] ...    Additional arguments. Depending on the format string, the function may expect a sequence of
 *                   additional arguments, each containing a value to be used to replace a format specifier in the
 *                   #format string.
 *
 * \note      There should be at least as many of the additional arguments as the number of values specified in
 *            the format specifiers. Additional arguments are ignored by the function.
 *
 * \return  On success, the total number of characters written is returned.
 */
extern int target_printf(const char * restrict format, ...);


/*!
 * \brief     Print a hex string in human readable format.
 *
 * \details   This function outputs the data contained in the #hex_data variable in a human readable form.
 *            Each byte will be output as 2 characters where each character is 0 -9 or A - F.
 *
 * \param[in] hex_data The data that needs to be output
 * \param[in] length   The number of bytes contained in the #hex_data array.
 */
extern void print_hex_string(uint8_t * const hex_data, size_t length);


#if defined(__cplusplus)
}
#endif

#endif /* _IID_TARGET_H_ */
