/*
 * Copyright 2021 Intrinsic ID B.V. All rights reserved.
 *
 * Usage of this software is permitted under a valid written license agreement
 * between you and Intrinsic ID B.V.
 */

/**
 * \file iidbroadkey.h
 *
 * \brief BK API header file.
 *
 * \details This header file defines the functional interface of BK's public API.
 */

#ifndef __IID_BROADKEY_H__
#define __IID_BROADKEY_H__

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef __KERNEL__
#include <linux/types.h>
#else
#include <stdint.h>
#endif

#include <stdbool.h>

/************************************************************************
*                        D E F I N I T I O N S                          *
*************************************************************************/

/** \brief BK configuration identifiers.
 */
#define BK_CONFIGURATION_PKI_ENABLED
#define BK_MAIN_CONFIGURATION_PKI_ENABLED

#define IID_CONFIGURATION_PKI_ENABLED
#define IID_MAIN_CONFIGURATION_PKI_ENABLED

/**
 * \name BK Parameter Size Definitions
 * \anchor bk_defgroup_size
 * @{
 */

/** \brief Root security strength of this BK configuration
 */
#define BK_SECURITY_SIZE_BITS               256

/** \brief Size in bytes of the SRAM PUF allocation
 */
#define BK_SRAM_PUF_SIZE_BYTES              1024

/** \brief Size in bytes of activation code buffer
 */
#define BK_AC_SIZE_BYTES                    968

/** \brief Size in bytes of the core unique identifier
 */
#define BK_CORE_UID_BYTES                   32

/** \brief Maximum size in bytes for this configuration, of a symmetric key used by BK
 */
#define BK_SYM_MAX_KEY_BYTES                32

/** \brief Maximum size in words for this configuration, of a symmetric key used by BK
 */
#define BK_SYM_MAX_KEY_WORDS                8

/** \brief Size in bytes of a symmetric key code capable of storing the maximum sized symmetric key for this configuration.
 */
#define BK_SYM_MAX_KEY_CODE_BYTES          (44 + BK_SYM_MAX_KEY_BYTES)

/** \brief Size in bytes of the header of a key code used to wrap external keys.
 */
#define BK_USER_KEY_CODE_NONKEY_BYTES      (44)



/** \brief Size in bytes, of an SECP256R1 elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_CURVE_SECP256R1_N_BYTES      32

/** \brief Size in words, of an SECP256R1 elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_CURVE_SECP256R1_N_WORDS      8

/** \brief Size in bytes, of an SECP256R1 elliptic curve field element that represents a coordinate of a curve point
 */
#define BK_ECC_CURVE_SECP256R1_P_BYTES      32

/** \brief Size in words, of an SECP256R1 elliptic curve field element that represents a coordinate of a curve point
 */
#define BK_ECC_CURVE_SECP256R1_P_WORDS      8

/** \brief Size in bytes, of an SECP256R1 elliptic curve private key
 */
#define BK_ECC_CURVE_SECP256R1_PRIVATE_KEY_BYTES      32

/** \brief Size in bytes, of an SECP256R1 elliptic curve public key
 */
#define BK_ECC_CURVE_SECP256R1_PUBLIC_KEY_BYTES       65

/** \brief Size in bytes, of an SECP256R1 elliptic curve signature
 */
#define BK_ECC_CURVE_SECP256R1_SIGNATURE_BYTES        64

/** \brief Size in bytes, of an SECP256R1 elliptic curve shared secret
 */
#define BK_ECC_CURVE_SECP256R1_SHARED_SECRET_BYTES    32

/** \brief Size in bytes, of an SECP256R1 elliptic curve static cryptogram header
 */
#define BK_ECC_CURVE_SECP256R1_STATIC_CRYPTOGRAM_HEADER_BYTES     144

/** \brief Size in bytes, of an SECP256R1 elliptic curve ephemeral cryptogram header
 */
#define BK_ECC_CURVE_SECP256R1_EPHEMERAL_CRYPTOGRAM_HEADER_BYTES  208

/** \brief Size in bytes, of an SECP256R1 elliptic curve private key info ASN.1 header
 */
#define BK_ECC_CURVE_SECP256R1_PRIVATE_KEY_INFO_DER_HEADER_BYTES     24

/** \brief Size in bytes, of an SECP256R1 elliptic curve public key info ASN.1 header
 */
#define BK_ECC_CURVE_SECP256R1_PUBLIC_KEY_INFO_DER_HEADER_BYTES     26

/** \brief Size in bytes, of an SECP256R1 elliptic curve sig value ASN.1 header
 */
#define BK_ECC_CURVE_SECP256R1_SIG_VALUE_DER_HEADER_BYTES           8

/** \brief Size in bytes, of an SECP256R1 elliptic curve signature ASN.1 header
 */
#define BK_ECC_CURVE_SECP256R1_SIGNATURE_DER_HEADER_BYTES           11

/** \brief Size in bytes, of an SECP224R1 elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_CURVE_SECP224R1_N_BYTES       28

/** \brief Size in words, of an SECP224R1 elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_CURVE_SECP224R1_N_WORDS       7

/** \brief Size in bytes, of an SECP224R1 elliptic curve field element that represents a coordinate of a curve point
 */
#define BK_ECC_CURVE_SECP224R1_P_BYTES       28

/** \brief Size in words, of an SECP224R1 elliptic curve field element that represents a coordinate of a curve point
 */
#define BK_ECC_CURVE_SECP224R1_P_WORDS       7

/** \brief Size in bytes, of an SECP224R1 elliptic curve private key
 */
#define BK_ECC_CURVE_SECP224R1_PRIVATE_KEY_BYTES      28

/** \brief Size in bytes, of an SECP224R1 elliptic curve public key
 */
#define BK_ECC_CURVE_SECP224R1_PUBLIC_KEY_BYTES       57

/** \brief Size in bytes, of an SECP224R1 elliptic curve signature
 */
#define BK_ECC_CURVE_SECP224R1_SIGNATURE_BYTES        56

/** \brief Size in bytes, of an SECP224R1 elliptic curve shared secret
 */
#define BK_ECC_CURVE_SECP224R1_SHARED_SECRET_BYTES    28

/** \brief Size in bytes, of an SECP224R1 elliptic curve static cryptogram header
 */
#define BK_ECC_CURVE_SECP224R1_STATIC_CRYPTOGRAM_HEADER_BYTES     136

/** \brief Size in bytes, of an SECP224R1 elliptic curve ephemeral cryptogram header
 */
#define BK_ECC_CURVE_SECP224R1_EPHEMERAL_CRYPTOGRAM_HEADER_BYTES  192

/** \brief Size in bytes, of an SECP224R1 elliptic curve private key info ASN.1 header
 */
#define BK_ECC_CURVE_SECP224R1_PRIVATE_KEY_INFO_DER_HEADER_BYTES     21

/** \brief Size in bytes, of an SECP224R1 elliptic curve public key info ASN.1 header
 */
#define BK_ECC_CURVE_SECP224R1_PUBLIC_KEY_INFO_DER_HEADER_BYTES     23

/** \brief Size in bytes, of an SECP224R1 elliptic curve sig value ASN.1 header
 */
#define BK_ECC_CURVE_SECP224R1_SIG_VALUE_DER_HEADER_BYTES           8

/** \brief Size in bytes, of an SECP224R1 elliptic curve signature ASN.1 header
 */
#define BK_ECC_CURVE_SECP224R1_SIGNATURE_DER_HEADER_BYTES           11

/** \brief Size in bytes, of an SECP192R1 elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_CURVE_SECP192R1_N_BYTES       24

/** \brief Size in words, of an SECP192R1 elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_CURVE_SECP192R1_N_WORDS       6

/** \brief Size in bytes, of an SECP192R1 elliptic curve field element that represents a coordinate of a curve point
 */
#define BK_ECC_CURVE_SECP192R1_P_BYTES       24

/** \brief Size in words, of an SECP192R1 elliptic curve field element that represents a coordinate of a curve point
 */
#define BK_ECC_CURVE_SECP192R1_P_WORDS       6

/** \brief Size in bytes, of an SECP192R1 elliptic curve private key
 */
#define BK_ECC_CURVE_SECP192R1_PRIVATE_KEY_BYTES      24

/** \brief Size in bytes, of an SECP192R1 elliptic curve public key
 */
#define BK_ECC_CURVE_SECP192R1_PUBLIC_KEY_BYTES       49

/** \brief Size in bytes, of an SECP192R1 elliptic curve signature
 */
#define BK_ECC_CURVE_SECP192R1_SIGNATURE_BYTES        48

/** \brief Size in bytes, of an SECP192R1 elliptic curve shared secret
 */
#define BK_ECC_CURVE_SECP192R1_SHARED_SECRET_BYTES    24

/** \brief Size in bytes, of an SECP192R1 elliptic curve static cryptogram header
 */
#define BK_ECC_CURVE_SECP192R1_STATIC_CRYPTOGRAM_HEADER_BYTES     128

/** \brief Size in bytes, of an SECP192R1 elliptic curve ephemeral cryptogram header
 */
#define BK_ECC_CURVE_SECP192R1_EPHEMERAL_CRYPTOGRAM_HEADER_BYTES  176

/** \brief Size in bytes, of an SECP192R1 elliptic curve private key info ASN.1 header
 */
#define BK_ECC_CURVE_SECP192R1_PRIVATE_KEY_INFO_DER_HEADER_BYTES     24

/** \brief Size in bytes, of an SECP192R1 elliptic curve public key info ASN.1 header
 */
#define BK_ECC_CURVE_SECP192R1_PUBLIC_KEY_INFO_DER_HEADER_BYTES     26

/** \brief Size in bytes, of an SECP192R1 elliptic curve sig value ASN.1 header
 */
#define BK_ECC_CURVE_SECP192R1_SIG_VALUE_DER_HEADER_BYTES           8

/** \brief Size in bytes, of an SECP192R1 elliptic curve signature ASN.1 header
 */
#define BK_ECC_CURVE_SECP192R1_SIGNATURE_DER_HEADER_BYTES           11

/** \brief Maximum size in bytes for this configuration, of an elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_MAX_CURVE_SIZE_N_BYTES       32

/** \brief Maximum size in words for this configuration, of an elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_MAX_CURVE_SIZE_N_WORDS       8

/** \brief Maximum size in bytes for this configuration, of an elliptic curve field element that represents a coordinate of
 * a curve point
 */
#define BK_ECC_MAX_CURVE_SIZE_P_BYTES       32

/** \brief Maximum size in words for this configuration, of an elliptic curve field element that represents a coordinate of
 *  a curve point
 */
#define BK_ECC_MAX_CURVE_SIZE_P_WORDS       8

/** \brief Maximum size in bytes for this configuration, of an elliptic curve field element that represents a curve point
 */
#define BK_ECC_MAX_CURVE_POINT_BYTES        64

/** \brief Maximum size in words for this configuration, of an elliptic curve field element that represents a curve point
 */
#define BK_ECC_MAX_CURVE_POINT_WORDS        16

/** \brief Maximum size in bytes for this configuration, of an elliptic curve sig value ASN.1 header
 */
#define BK_ECC_MAX_SIG_VALUE_DER_HEADER_BYTES           8

/** \brief Maximum size in bytes for this configuration, of an elliptic curve signature ASN.1 header
 */
#define BK_ECC_MAX_SIGNATURE_DER_HEADER_BYTES           11

/** \brief Maximum size in bytes for this configuration, of an elliptic curve public key info ASN.1 header
 */
#define BK_ECC_MAX_PUBLIC_KEY_INFO_DER_HEADER_BYTES     26

/** \brief Maximum size in bytes for this configuration, of an elliptic curve private key info ASN.1 header
 */
#define BK_ECC_MAX_PRIVATE_KEY_INFO_DER_HEADER_BYTES    24

/** \brief Maximum size in bytes for this configuration, of an elliptic curve scalar, smaller than the order of the curve
 */
#define BK_ECC_MAX_CURVE_SIZE_BYTES         BK_ECC_MAX_CURVE_SIZE_N_BYTES

/** \brief Maximum size in bytes for this configuration, of an elliptic curve field element that represents a coordinate of
 *  a curve point
 */
#define BK_ECC_MAX_CURVE_SIZE_WORDS         BK_ECC_MAX_CURVE_SIZE_N_WORDS

/** \brief Size in bytes for this configuration, of the store capable of holding the maximum sized elliptic curve private
 *  key
 */
#define BK_ECC_PRIVATE_KEY_BYTES  BK_ECC_MAX_CURVE_SIZE_N_BYTES

/** \brief Size in bytes for this configuration, of the store capable of holding the maximum sized elliptic curve public
 *  key
 */
#define BK_ECC_PUBLIC_KEY_BYTES  BK_ECC_MAX_CURVE_POINT_BYTES

/** \brief Size in bytes for this configuration, of the store capable of holding the maximum sized elliptic curve x9.62
 *  public key
 */
#define BK_ECC_STD_PUBLIC_KEY_BYTES  (1 + BK_ECC_MAX_CURVE_POINT_BYTES)

/** \brief Size in bytes for this configuration, of the store capable of holding the maximum sized elliptic curve ECDSA
 *  signature
 */
#define BK_ECC_SIGNATURE_BYTES  BK_ECC_MAX_CURVE_POINT_BYTES

/** \brief Size in bytes for this configuration, of the store capable of holding the maximum sized elliptic curve ECDH
 *  shared secret
 */
#define BK_ECC_SHARED_SECRET_BYTES  BK_ECC_MAX_CURVE_SIZE_P_BYTES

/** \brief Size in words for this configuration, of the store capable of holding the maximum sized elliptic curve private
 *  key
 */
#define BK_ECC_PRIVATE_KEY_WORDS  BK_ECC_MAX_CURVE_SIZE_N_WORDS

/** \brief Size in words for this configuration, of the store capable of holding the maximum sized elliptic curve public
 *  key
 */
#define BK_ECC_PUBLIC_KEY_WORDS   BK_ECC_MAX_CURVE_POINT_WORDS

/** \brief Size in bytes for this configuration, of the store capable of holding the maximum sized elliptic curve ECDSA
 *  signature
 */
#define BK_ECC_SIGNATURE_WORDS  BK_ECC_MAX_CURVE_POINT_WORDS

/** \brief Size in bytes for this configuration, of the store capable of holding the maximum sized elliptic curve ECDH
 *  shared secret
 */
#define BK_ECC_SHARED_SECRET_WORDS  BK_ECC_MAX_CURVE_SIZE_P_WORDS


/** \brief Size in bytes for this configuration, of the byte array capable of holding the maximum sized elliptic curve
 *  ECDSA ASN.1 DER encoded sig value
 */
#define BK_ECC_DER_SIG_VALUE_BYTES  (BK_ECC_MAX_SIG_VALUE_DER_HEADER_BYTES + BK_ECC_SIGNATURE_BYTES)

/** \brief Size in bytes for this configuration, of the byte array capable of holding the maximum sized elliptic curve
 *  ECDSA ASN.1 DER encoded signature
 */
#define BK_ECC_DER_SIGNATURE_BYTES  (BK_ECC_MAX_SIGNATURE_DER_HEADER_BYTES + BK_ECC_SIGNATURE_BYTES)

/** \brief Size in bytes for this configuration, of the byte array capable of holding the maximum sized elliptic curve
 *  ECDSA ASN.1 DER encoded public key info
 */
#define BK_ECC_DER_PUBLIC_KEY_INFO_BYTES  (BK_ECC_MAX_PUBLIC_KEY_INFO_DER_HEADER_BYTES + BK_ECC_STD_PUBLIC_KEY_BYTES)

/** \brief Size in bytes for this configuration, of the byte array capable of holding the maximum sized elliptic curve
 *  ECDSA ASN.1 DER encoded public key info
 */
#define BK_ECC_DER_PRIVATE_KEY_INFO_BYTES  (BK_ECC_MAX_PRIVATE_KEY_INFO_DER_HEADER_BYTES + BK_ECC_PRIVATE_KEY_BYTES + BK_ECC_STD_PUBLIC_KEY_BYTES)

/** \brief Size in bytes of the header of an elliptic curve key code
 */
#define BK_ECC_KEY_CODE_HEADER_BYTES   48
/** \brief Size in words of the header of an elliptic curve key code
 */
#define BK_ECC_KEY_CODE_HEADER_WORDS   12

/** \brief Size in bytes for this configuration, of an elliptic curve private key code capable of storing the maximum sized
 *  private key
 */
#define BK_ECC_PRIVATE_KEY_CODE_SIZE_BYTES  (4 * (BK_ECC_KEY_CODE_HEADER_WORDS + BK_ECC_MAX_CURVE_SIZE_N_WORDS))

/** \brief Size in bytes for this configuration, of the representation capable of holding the maximum sized elliptic curve
 *  public key
 */
#define BK_ECC_PUBLIC_KEY_PACK_BYTES        (BK_ECC_MAX_CURVE_POINT_BYTES)

/** \brief Size in bytes for this configuration, of an elliptic curve public key code capable of storing the maximum sized
 *  public key
 */
#define BK_ECC_PUBLIC_KEY_CODE_SIZE_BYTES   (4 * (BK_ECC_KEY_CODE_HEADER_WORDS + BK_ECC_MAX_CURVE_POINT_WORDS))

/** \brief Size in bytes of the fixed part of the header of a cryptogram
 */
#define BK_HYBRID_CRYPTOGRAM_HEADER_BYTES    80

/** \brief maximum size in bytes for this configuration, of the header of a static key pair cryptogram
 */
#define BK_HYBRID_MAX_STATIC_CRYPTOGRAM_HEADER_SIZE_BYTES         (BK_HYBRID_CRYPTOGRAM_HEADER_BYTES + 4 * (BK_ECC_MAX_CURVE_POINT_WORDS))

/** \brief maximum size in bytes for this configuration, of the header of an ephemeral key pair cryptogram
 */
#define BK_HYBRID_MAX_EPHEMERAL_CRYPTOGRAM_HEADER_SIZE_BYTES      (BK_HYBRID_CRYPTOGRAM_HEADER_BYTES + 8 * (BK_ECC_MAX_CURVE_POINT_WORDS))

/** \brief Size in bytes of the fixed part of the header of a cryptogram (legacy definition) */
#define BK_ECC_CRYPTOGRAM_HEADER_SIZE_BYTES       BK_HYBRID_CRYPTOGRAM_HEADER_BYTES

/**
@}
 */

/**
 * \name BK DER/X.509 Encoding Definitions
 * \anchor bk_defgroup_der
@{
 */
/*
 * DER length encoding supports unsigned integers with 1016 bits / 127 bytes,
 * thus with a length specification that has already a size of up to 127 bytes
 * to reduce buffer usage, this interface is limited to the size of uint64_t.
 *
 * Long integers represented as a uint8_t array can have full length
 */

 /** \brief Maximum size in bytes of a length integer in a DER encoding
  */
#define BK_DER_LENGTH_OCTETS              8

/** \brief Maximum size in bytes of the entire length field in a DER encoding
 */
#define BK_DER_MAX_LENGTH_FIELD           (1 + BK_DER_LENGTH_OCTETS)

/** \brief The maximum number of bytes in the (long integer) serial number of a self-signed certificate (SSC)
 */
#define BK_SSC_MAX_SERIAL_NUMBER_BYTES    20

/** \brief The maximum number of characters in the validity fields (notBefore, notAfter) generalised time value string of a
 *  self-signed certificate (SSC)
 */
#define BK_SSC_TIME_DIGITS                14

/** \brief The GeneralizedTime value of "99991231235959Z" used in case of not well defined validity fields (notBefore,
 *  notAfter) of a self-signed certificate
 */
#define BK_SSN_UNDEFINED_VALIDITY         "99991231235959Z"

/** \brief The maximum number of characters of CSR or self-signed certificate (SSC) subject distinguished name fields.
 *  This size limit is not due to memory restrictions, but to guard against ill formatted strings.
 */
#define BK_DER_MAX_SUBJECT_STR            64

/** \brief The maximum number of characters of CSR or self-signed certificate (SSC) OID strings. This size limit is not
 *  due to memory restrictions, but to guard against ill formatted strings.
 */
#define BK_DER_MAX_OID_STR                32


/**
 * @}
 */

/** \brief The function return type
 *  \details
 */
typedef uint8_t iid_return_t;

/**
 * \typedef bk_sym_key_type_t
 * \brief Symmetric key type
 * \enum bk_sym_key_type
 * \brief Enumeration of symmetric key types
 *
 * Enumerates the symmetric key types which can be generated by the bk_get_key() function. This key type also implies the size in bytes of the referred key, as indicated.
 */
typedef enum bk_sym_key_type
{
    /**
     * \brief 128-bit Symmetric Key
     *
     * \details A 128-bit key for symmetric cryptography algorithms:
     * - size in bytes of the key value: 16
     * - size in bytes of the bk_wrap()-generated key code: (16 + #BK_USER_KEY_CODE_NONKEY_BYTES)
     * - maximal security strength for operations with this key: 128-bit
     */
    BK_SYM_KEY_TYPE_128 = 0x00,
    /**
     * \brief 192-bit Symmetric Key
     *
     * \details A 192-bit key for symmetric cryptography algorithms:
     * - size in bytes of the key value: 24
     * - size in bytes of the bk_wrap()-generated key code: (24 + #BK_USER_KEY_CODE_NONKEY_BYTES)
     * - maximal security strength for operations with this key: 192-bit
     */
    BK_SYM_KEY_TYPE_192 = 0x01,
    /**
     * \brief 256-bit Symmetric Key
     *
     * \details A 256-bit key for symmetric cryptography algorithms:
     * - size in bytes of the key value: 32
     * - size in bytes of the bk_wrap()-generated key code: (32 + #BK_USER_KEY_CODE_NONKEY_BYTES)
     * - maximal security strength for operations with this key: 256-bit
     */
    BK_SYM_KEY_TYPE_256 = 0x02
} bk_sym_key_type_t;

/**
 * \typedef bk_key_source_id_t
 * \brief Key source identifier
 * \enum bk_key_source_id
 * \brief Enumeration of key sources
 *
 * Enumerates the allowed sources from which a generic, symmetric or elliptic curve private key can be generated for use
 * with cryptographic functions..
 */
typedef enum bk_key_source_id
{
    /**
     * \brief Key derived from PUF
     *
     * \details The key is derived in a direct line from the SRAM PUF's device unique start-up data. Keys derived in this
     * way can always be exactly rederived by calling cryptographic functions with the same arguments in the same
     * cryptographic context, and on the same device.
     */
    BK_KEY_SOURCE_PUF_DERIVED = 0x00,
    /**
     * \brief Randomly generated key
     *
     * \details The key is randomly generated using BK's internal cryptographically secure random number generator which is
     *  seeded by entropy coming from the device noise. Keys generated in this way cannot be rederived by cryptographic
     *  functions.
     */
    BK_KEY_SOURCE_RNG_DERIVED = 0x01,
    /**
     * \brief Key provided by user
     *
     * \details The key is an external value which is provided by the calling application to the bk_create_private_key()
     * function.
     */
    BK_KEY_SOURCE_USER_PROVIDED = 0x02,
} bk_key_source_id_t;

/**
 * \typedef bk_ecc_key_source_t
 * \brief Defines the allowed sources from which an elliptic curve private key can be generated for use with bk_create_private_key().
 */
typedef bk_key_source_id_t bk_ecc_key_source_t;

/**
 * \name BK ECC Key Source ID Definitions (legacy)
 * \anchor bk_defgroup_ecckeysource
@{
 */

/** \brief Equivalent to \ref BK_KEY_SOURCE_PUF_DERIVED
 */
#define BK_ECC_KEY_SOURCE_PUF_DERIVED     ((bk_ecc_key_source_t)0x00)

/** \brief Equivalent to \ref  BK_KEY_SOURCE_RNG_DERIVED
 */
#define BK_ECC_KEY_SOURCE_RANDOM          ((bk_ecc_key_source_t)0x01)

/** \brief Equivalent to \ref BK_KEY_SOURCE_USER_PROVIDED
 */
#define BK_ECC_KEY_SOURCE_USER_PROVIDED   ((bk_ecc_key_source_t)0x02)

/**
@}
 */

/**
 * \typedef bk_key_purpose_t
 * \brief Key purpose
 *
* Generic key purpose type which is further specialized as a ::bk_ecc_key_purpose_t.
 */
typedef uint8_t bk_key_purpose_t;

/**
 * \typedef bk_ecc_curve_t
 * \brief Named elliptic curve
 * \enum bk_ecc_curve
 * \brief Enumeration of named elliptic curves
 *
 * Enumerates the named elliptic curves which can be recognized by BK's elliptic curve functions. The used curve also
 * implies the size in bytes of several input and output parameters of these functions, as indicated.
 */
typedef enum bk_ecc_curve
{
    /**
     * \brief Curve NIST-P192
     *
     * \details The elliptic-curve cryptosystem specified by the NIST-P192/secp192r1 domain parameters:
     *
     * - BK curve ID: 0x00
     * - size in bytes of private keys: 24
     * - size in bytes of public keys: 49
     * - size in bytes of ECDSA signatures: 48
     * - size in bytes of ECDH shared secrets: 24
     * - size in bytes of ECIES header: 49
     * - maximal security strength for operations over this curve: 96-bit
     */
    BK_ECC_CURVE_NIST_P192 = 0x00,
    /**
     * \brief Curve NIST-P224
     *
     * \details The elliptic-curve cryptosystem specified by the NIST-P224/secp224r1 domain parameters:
     *
     * - BK curve ID: 0x01
     * - size in bytes of private keys: 28
     * - size in bytes of public keys: 57
     * - size in bytes of ECDSA signatures: 56
     * - size in bytes of ECDH shared secrets: 28
     * - size in bytes of ECIES header: 57
     * - maximal security strength for operations over this curve: 112-bit
     */
    BK_ECC_CURVE_NIST_P224 = 0x01,
    /**
     * \brief Curve NIST-P256
     *
     * \details The elliptic-curve cryptosystem specified by the NIST-P256/secp256r1 domain parameters:
     *
     * - BK curve ID: 0x02
     * - size in bytes of private keys: 32
     * - size in bytes of public keys: 65
     * - size in bytes of ECDSA signatures: 64
     * - size in bytes of ECDH shared secrets: 32
     * - size in bytes of ECIES header: 65
     * - maximal security strength for operations over this curve: 128-bit
     */
    BK_ECC_CURVE_NIST_P256 = 0x02,
} bk_ecc_curve_t;

/**
 * \typedef bk_ecc_private_key_t
 * \brief Elliptic curve private key.
 *
 * \details Defines the private key type capable of storing the the maximum sized private key for this configuration.
 */
typedef uint8_t bk_ecc_private_key_t[BK_ECC_PRIVATE_KEY_BYTES];

/**
 * \typedef bk_ecc_key_purpose_t
 * \brief Elliptic curve key purpose
 *
 * \details Defines the allowed purposes which can be assigned to an elliptic curve private or public key. It is stored
 * alongside the key in the corresponding public- or private key code.
 */
typedef bk_key_purpose_t bk_ecc_key_purpose_t;

/**
 * \name BK Key Purpose Flag Definitions
 * \anchor bk_defgroup_keypurpose
@{
 */

/** \brief Elliptic curve key (code)s marked with this purpose flag can be used for elliptic curve key agreement (ECDH)
 *  and en/decryption functions (ECIES, cryptogram).
 */
#define BK_ECC_KEY_PURPOSE_ECDH               ((bk_ecc_key_purpose_t)0x01)

/** \brief Elliptic curve key (code)s marked with this purpose flag can be used for elliptic curve signature (ECDSA)
 * generation and verification functions (ECDSA-sign, ECDSA-verify, CSR, self-signed certificate).
 */
#define BK_ECC_KEY_PURPOSE_ECDSA              ((bk_ecc_key_purpose_t)0x02)

/** \brief This is the combination of the ECDH and ECDSA flags. Elliptic curve key (code)s marked with this purpose flag
 * can be used for elliptic curve key agreement (ECDH) en/decryption functions (ECIES, cryptogram) as well as signature
 * (ECDSA) generation and verification functions (ECDSA-sign, ECDSA-verify, CSR, self-signed certificate)
 */
#define BK_ECC_KEY_PURPOSE_ECDH_AND_ECDSA     ((bk_ecc_key_purpose_t)(0x01 | 0x02))

/**
@}
 */

/**
 * \typedef bk_ecc_std_public_key_t
 * \brief Elliptic curve public key.
 *
 * \details Defines the  public key type capable of storing the the maximum sized public key in X9.62 binary format for this configuration.
 */
typedef uint8_t bk_ecc_std_public_key_t[BK_ECC_STD_PUBLIC_KEY_BYTES];

/**
 * \typedef bk_ecc_signature_t
 * \brief Elliptic curve ECDSA signature.
 *
 * \details Defines the ECDSA signature type capable of storing the the maximum sized signature for this configuration.
 */
typedef uint8_t bk_ecc_signature_t[BK_ECC_SIGNATURE_BYTES];

/**
 * \typedef bk_ecc_shared_secret_t
 * \brief Elliptic curve ECDH shared secret.
 *
 * \details Defines the ECDH shared secret type capable of storing the the maximum sized shared secret for this configuration.
 */
typedef uint8_t bk_ecc_shared_secret_t[BK_ECC_SHARED_SECRET_BYTES];

/**
 * \typedef bk_ecc_private_key_code_t
 * \brief Elliptic curve private key code.
 *
 * \details Defines the private key code type capable of storing the the maximum sized private key for this configuration.
 */
typedef uint8_t bk_ecc_private_key_code_t[BK_ECC_PRIVATE_KEY_CODE_SIZE_BYTES];

/**
 * \typedef bk_ecc_public_key_code_t
 * \brief Elliptic curve public key code.
 *
 * \details Defines the public key code type capable of storing the the maximum sized public key for this configuration.
 */
typedef uint8_t bk_ecc_public_key_code_t[BK_ECC_PUBLIC_KEY_CODE_SIZE_BYTES];


/**
 * \typedef bk_ecc_cryptogram_type_t
 * \brief Elliptic curve cryptogram type
 * \enum bk_ecc_cryptogram_type
 * \brief Enumeration of elliptic curve cryptogram types
 *
 * Enumerates the defined cryptogram types used in bk_generate_cryptogram() and returned by bk_process_cryptogram().
 */
typedef enum bk_ecc_cryptogram_type
{
    /**
     * \brief Cryptogram type using static key pairs on both sides
     *
     * \details Cryptogram type using static elliptic curve key pairs on both sending and receiving side:
     * - offers message confidentiality, message integrity, source authentication, replay protection
     * - does \b not offer forward secrecy, non-repudiation
     * - size in bytes of cryptogram = size in bytes of plaintext + #BK_ECC_CRYPTOGRAM_HEADER_SIZE_BYTES + size in bytes of public key according to [curve](\ref bk_ecc_curve) - 1
     */
    BK_ECC_CRYPTOGRAM_TYPE_ECDH_STATIC = 0x00,

    /**
     * \brief Cryptogram type using an ephemeral sender key pair.
     *
     * \details Cryptogram type using an ephemeral key pair on the sending side, and a static key pair on the receiving side:
     * - offers message confidentiality, message integrity, source authentication, replay protection, \b and forward secrecy against compromise of the sender's private key
     * - does \b not offer non-repudiation
     * - size in bytes of cryptogram = size in bytes of plaintext + #BK_ECC_CRYPTOGRAM_HEADER_SIZE_BYTES + (2* size in bytes of public key according to [curve](\ref bk_ecc_curve)) - 2
     */
    BK_ECC_CRYPTOGRAM_TYPE_ECDH_EPHEMERAL = 0x01
} bk_ecc_cryptogram_type_t;

/* stdint.h, stdbool.h have been included at the top CMAKE file */

/**
 * \typedef bk_der_sig_value_t
 * \brief DER-encoded ECDSA signature following the \p ECDSA-Sig-Value ASN.1 syntax of
 * [RFC 3279](https://tools.ietf.org/html/rfc3279)
 *
 * \details Defines the byte array type capable of storing the the maximum sized DER-encoded ECDSA-Sig-Value for this configuration.
 */
typedef uint8_t bk_der_sig_value_t[BK_ECC_DER_SIG_VALUE_BYTES];

/**
 * \typedef bk_der_signature_t
 * \brief Elliptic curve ASN.1 DER encoded signature.
 *
 * \details Defines the byte array type capable of storing the the maximum sized ECDSA ASN.1 DER encoded signature for this
 * configuration.
 */
typedef uint8_t bk_der_signature_t[BK_ECC_DER_SIGNATURE_BYTES];

/**
 * \typedef bk_der_public_key_info_t
 * \brief DER-encoded public key following the \p SubjectPublicKeyInfo ASN.1 syntax of
 * [RFC 5480](https://tools.ietf.org/html/rfc5480)
 *
 * \details Defines the byte array type capable of storing the the maximum sized DER-encoded SubjectPublicKeyInfo for this
 * configuration.
 */
typedef uint8_t bk_der_public_key_info_t[BK_ECC_DER_PUBLIC_KEY_INFO_BYTES];

/**
 * \typedef bk_der_private_key_info_t
 * \brief DER-encoded elliptic-curve private key following the \p ECPrivateKey ASN.1 syntax of
 * [RFC 5915](https://tools.ietf.org/html/rfc5915)
 *
 * \details Defines the byte array type capable of storing the the maximum sized DER-encoded elliptic-curve private key for
 * this configuration.
 */
typedef uint8_t bk_der_private_key_info_t[BK_ECC_DER_PRIVATE_KEY_INFO_BYTES];

/**
 * \typedef bk_certificate_subject_t
 * \brief Certificate (subject) distinguished name
 */
typedef struct bk_certificate_subject_s bk_certificate_subject_t;

/**
 * \struct bk_certificate_subject_s
 * \brief Structure with certificate (subject) distinguished name fields
 *
 * \details Structure containing the supported distinguished name fields (X.500) which can be defined for a certificate
 * subject used in CSR and self-signed certificate (SSC) generation. The name fields are expected as C-style \0-terminated
 * character strings. The maximim character length of each of these fields is limited to #BK_DER_MAX_SUBJECT_STR.
 */
struct bk_certificate_subject_s {
   /**
    * \brief String describing the subject's \b countryName field (OID 2.5.4.6)
    */
    const char      * subject_c;
   /**
    * \brief String describing the subject's \b organisationName field (OID 2.5.4.10)
    */
    const char      * subject_o;
   /**
    * \brief String describing the subject's \b commonName field (OID 2.5.4.3)
    */
    const char      * subject_cn;
   /**
    * \brief String describing the subject's \b serialNumber field (OID 2.5.4.5)
    */
    const char      * subject_sn;
};


/****************************************************************************
*                      P U B L I C  I N T E R F A C E                       *
*****************************************************************************/

/**
 * \name BK Return Code Bases
 * \anchor bk_defgroup_return_base
@{
 */

/**
 * \brief Generic return codes base
 * \details Value used internally as the base value for all generic return codes.
 */
#define IID_RETURN_BASE              0x10

/**
 * \brief BK basic operation return codes base
 * \details Value used internally as the base value for all BK's basic operation specific return codes.
 */
#define IID_RETURN_BK                0x20

/**
 * \brief BK elliptic curve operations return codes base
 * \details Value used internally as the base value for all BK's ECC specific return codes.
 */
#define IID_RETURN_ECC               0x30

/**
 * \brief BK PKI operations return codes base
 * \details Value used internally as the base value for all BK's PKI specific return codes.
 */
#define IID_RETURN_PKI               0x40

/**
@}
 */

/**
 * \name BK Generic Return Codes
 * \anchor bk_defgroup_return_generic
@{
 */

/**
 * \brief Busy executing
 * \details Value indicates that the execution of the called function has not terminated.
 */
#define IID_BUSY                    0xFF

/**
 * \brief Successful execution
 * \details Value indicating the successful execution of the called function. In case of value verification functions
 * (e.g. MAC verify, or signature verify, etc.), this return code also implies a successful verification.
 */
#define IID_SUCCESS                  0x00

/**
 * \brief Function call not allowed in the current state
 * \details Value indicating that the given function call is not allowed in the current state of BK.
 */
#define IID_NOT_ALLOWED              (IID_RETURN_BASE + 0x00)

/**
 * \brief Invalid function parameters
 * \details Value indicating that at least one of the parameters passed as argument to the function call has an invalid form
 * and/or content. This also occurs when one of the required output buffers contains a NULL pointer, or when one of the
 * provided length parameters is not long enough.
 */
#define IID_INVALID_PARAMETERS       (IID_RETURN_BASE + 0x01)

/**
 * \brief No free handle available
 * \details Value indicating that a handle cannot be allocated for the specified streaming interface.
 */
#define IID_NO_FREE_HANDLE           (IID_RETURN_BASE + 0x02)

/**
 * \brief Invalid streaming interface handle
 * \details Value indicating that the streaming interface handle provided to this function does not point to an existing and
 * opened streaming interface, or that a call to open a new streaming interface could not allocate sufficient memory for
 * creating it.
 */
#define IID_INVALID_HANDLE           (IID_RETURN_BASE + 0x03)

/**
 * \brief Invalid streaming interface context
 * \details Value indicating that the context pointed too by the provided handle does not exist, has been closed, or is
 * corrupted.
 */
#define IID_INVALID_CONTEXT          (IID_RETURN_BASE + 0x04)

/**
 * \brief Authentication of streaming interface data failed
 * \details Value indicating that a call to close a streaming interface with an authentication verification (e.g. a MAC or
 * ECIES stream) ends with a data authentication failure of the processed stream, thus a sign of data corruption.
 */
#define IID_INVALID_MAC              (IID_RETURN_BASE + 0x05)

/**
 * \brief Program data error
 * \details Value indicating that the internal program data is corrupted or undetermined.
 */
#define IID_DATA_EXCEPTION           (IID_RETURN_BASE + 0x0D)

/**
 * \brief Program flow error
 * \details Value indicating that the internal program state is corrupted or undetermined.
 */
#define IID_PROGRAM_EXCEPTION        (IID_RETURN_BASE + 0x0E)

/**
 * \brief Device integrity error
 * \details Value indicating that the internal device state is corrupted or undetermined.
 */
#define IID_DEVICE_EXCEPTION        (IID_RETURN_BASE + 0x0F)

/**
@}
 */

/**
 * \name BK Specific Return Codes
 * \anchor bk_defgroup_return_specific
@{
 */

/**
 * \brief Error startup data
 * \details Value indicating that the appointed SRAM address does not contain qualitative start-up data which can be used as
 * an SRAM PUF by BK.\n \b Note: this return value with high likelihood indicates a blocking error, since one is not able to
 * get out of the current state (recalling the operation that returned this value will likely just return the same error
 * code). A device repower is anyway required to get out of this situation, in combination with a resolution of the cause of
 * this problem, if possible.
 */
#define IID_ERROR_STARTUP_DATA       (IID_RETURN_BK + 0x00)

/**
 * \brief Invalid activation code
 * \details Value indicating that the activation code provided to bk_start() is not a valid activation code for this device,
 * i.e. it was not generated by a successful bk_enroll() call on the same device.
 */
#define IID_INVALID_AC               (IID_RETURN_BK + 0x01)

/**
 * \brief Invalid key code
 * \details Value indicating that the key code provided, to bk_unwrap() or bk_create_symmetric_key(), is not a valid key
 * code for this device, i.e. it was not generated by a successful bk_wrap() call on the same device in the same
 * cryptographic context.
 */
#define IID_INVALID_KEY_CODE         (IID_RETURN_BK + 0x02)

/**
 * \brief Presented symmetric key code is not allowed to be used for this operation
 * \details Value indicating that the provided symmetric key code input does not have the right [key purpose flags]
 * (bk_defgroup_keypurpose) set for being used by the called function.
 */
#define IID_SYM_NOT_ALLOWED          (IID_RETURN_BK + 0x03)

/**
 * \brief Invalid symmetric key code
 * \details Value indicating that the provided symmetric key code input is not a valid symmetric key code for the called
 * function on this device, i.e. it was not generated by bk_create_symmetric_key() on this device.
 */
#define IID_INVALID_SYMMETRIC_KEY_CODE (IID_RETURN_BK + 0x04)

/**
@}
 */

/**
 * \name BK ECC Specific Return Codes
 * \anchor bk_defgroup_return_ecc
@{
 */

/**
 * \brief Invalid private key value
 * \details Value indicating that an externally provided private key input value (to bk_create_private_key() or
 * bk_derive_public_key()) is an invalid private key for the specified elliptic curve.
 */
#define IID_INVALID_PRIVATE_KEY      (IID_RETURN_ECC + 0x00)

/**
 * \brief Invalid public key value
 * \details Value indicating that the provided public key input value (to bk_import_public_key()) is an invalid public key
 * for the specified elliptic curve.
 */
#define IID_INVALID_PUBLIC_KEY       (IID_RETURN_ECC + 0x01)

/**
 * \brief Invalid signature
 * \details Value indicating that the signature input provided to bk_ecdsa_verify() is not a valid signature for the
 * provided message under the provided public key.
 */
#define IID_INVALID_SIGNATURE        (IID_RETURN_ECC + 0x02)

/**
 * \brief Invalid public key code
 * \details Value indicating that the provided public key code input is not a valid public key code for the called function
 * on this device, i.e. it was not generated by bk_compute_public_from_private_key() or bk_import_public_key() on this
 * device.
 */
#define IID_INVALID_PUBLIC_KEY_CODE  (IID_RETURN_ECC + 0x03)

/**
 * \brief Invalid private key code
 * \details Value indicating that the provided private key code input is not a valid private key code for the called
 * function on this device, i.e. it was not generated by bk_create_private_key() on this device.
 */
#define IID_INVALID_PRIVATE_KEY_CODE (IID_RETURN_ECC + 0x04)

/**
 * \brief Presented elliptic curve key code is not allowed to be used for this operation
 * \details Value indicating that the provided private and/or public key code inputs do not have the right
 * [key purpose flags](bk_defgroup_keypurpose) set for being used by the called function.
 */
#define IID_ECC_NOT_ALLOWED          (IID_RETURN_ECC + 0x05)

/**
 * \brief Presented cryptogram counter is invalid
 * \details Value indicating that the \p counter64 input provided to bk_generate_cryptogram() results in a counter overflow,
 * or that the \p counter64 input provided to bk_process_cryptogram() is not smaller than or equal to the counter value
 * contained in the received cryptogram. The latter indicates that there is an attempt to process a cryptogram that is the
 * same or older than a previously processed one.
 */
#define IID_INVALID_COUNTER          (IID_RETURN_ECC + 0x06)

/**
 * \brief Invalid cryptogram
 * \details Value indicating that the provided cryptogram input to bk_process_cryptogram() or
 * bk_get_public_key_from_cryptogram() is not a valid cryptogram, e.g. it has the wrong type, format, or its integrity was
 * compromised.
 */
#define IID_INVALID_CRYPTOGRAM       (IID_RETURN_ECC + 0x07)

/**
 * \brief Invalid sender of cryptogram
 * \details Value indicating that the provided public key code to bk_process_cryptogram() does not match the public key of
 * the cryptogram's sender, i.e. the cryptogram did not originate from the expected source.
 */
#define IID_INVALID_SENDER           (IID_RETURN_ECC + 0x08)

/**
 * \brief Curve mismatch
 * \details Value indicating that the simultaneously provided private and public key code to a function (to
 * bk_generate_cryptogram(), bk_process_cryptogram() or bk_ecdh_shared_secret()) do not match the same elliptic curve.
 */
#define IID_CURVE_MISMATCH           (IID_RETURN_ECC + 0x09)

/**
@}
 */

/**
 * \name BK PKI Specific Return Codes
 * \anchor bk_defgroup_return_pki
@{
 */

/**
 * \brief Insufficient PKI output buffer space
 * \details The provided output buffer for the requested PKI structure (CSR of self-signed certificate) is too small to hold
 * all data.
 */
#define IID_PKI_BUFFER_TOO_SMALL    (IID_RETURN_PKI + 0x00)

/**
 * \brief Invalid DER input
 * \details The provided DER encoded byte stream is either corrupted or holds unsupported data or method indicators.
 */
#define IID_PKI_INVALID_DER_INPUT    (IID_RETURN_PKI + 0x01)

/**
@}
 */

/**
 * \name BK Core Functions
 * \anchor bk_funcgroup_core
@{
 */

/**
 * \brief Gets software product and version information
 *
 * \details This function can be used to get the exact name, version and patch number of the software module.
 *
 * \pre A call to this function is allowed in all states of BK.
 *
 * \post A call to this function never changes the operational state of BK.
 *
 * \param[out] product_id    Pointer to an output buffer with a size of 1 byte which will hold the product identifier
 *                           character
 * \param[out] major_version Pointer to an output buffer with a size of 1 byte which will hold the major software version
 * \param[out] minor_version Pointer to an output buffer with a size of 1 byte which will hold the minor software version
 * \param[out] patch         Pointer to an output buffer with a size of 1 byte which will hold the software patch number
 * \param[out] build_number  Pointer to an output buffer with a size of 1 byte which will hold the build number
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS otherwise
 */
iid_return_t bk_get_product_info(
    uint8_t * const product_id,
    uint8_t * const major_version,
    uint8_t * const minor_version,
    uint8_t * const patch,
    uint8_t * const build_number);



/**
* \brief Gets specific software product and version string
*
* \details This function can be used to get the exact name, flavor, commit and version.
*
* \pre A call to this function is allowed in all states of BK.
*
* \post A call to this function never changes the operational state of BK.
*
* \returns A version string
*/
const char *
    bk_get_version_string(void);

/**
 * \brief Initializes BK after power-up or reset
 *
 * \details This function is used to initialize the BK software module before use, after each device power-up or reset.
 * It points the BK module to the system's SRAM range which is reserved as SRAM PUF.
 *
 * \pre A call to this function is only allowed when BK is in the *Uninitialized* state. Called from any other operational
 * state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A successful call to this function changes the operational state of BK to the *Initialized* state.
 *
 * \param[inout] sram_puf      Pointer to physical SRAM PUF used by the module. The physical SRAM pointed to is both read
 *                             (the start-up data it contains is used as a PUF), and written (amongst other things to
 *                             condition it against silicon aging).\n \b Note: the address must be at a 32-bit boundary, so
 *                             the lowest two bits of the address must be 0.
 * \param[in]    sram_puf_size The size in bytes of available SRAM PUF that can be used by the software.\n \b Note: the size
 *                             of the SRAM must be (at least) #BK_SRAM_PUF_SIZE_BYTES bytes
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS, #IID_ERROR_STARTUP_DATA or #IID_NOT_ALLOWED otherwise
 */
iid_return_t bk_init(
          uint8_t  * const sram_puf,
    const uint16_t         sram_puf_size);


/**
 * \brief Starts an existing cryptographic context for BK
 *
 * \details This function is used to re-instantiate a cryptographic context of BK based on a provided activation code which
 * was earlier generated by bk_enroll(). It is the responsibility of the calling software to reliably store an activation
 * code after bk_enroll(), and retrieve it before bk_start(). Once a cryptographic context is instantiated, BK's
 * cryptographic functionality becomes available
 *
 * \pre A call to this function is only allowed when BK is in the *Initialized* or *Stopped* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A successful call to this function (return code equals #IID_SUCCESS) changes the operational state of BK to the
 * *Started* state.
 *
 * \param[in] activation_code Pointer to an input buffer of byte size #BK_AC_SIZE_BYTES which holds the activation code.
 * \n \b Note: the address must be at a 32-bit boundary, so the lowest two bits of the address must be 0.
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS, #IID_INVALID_AC or #IID_NOT_ALLOWED otherwise
 */
iid_return_t bk_start(
    const uint8_t * const activation_code);

/**
 * \brief Stops the active cryptographic context of BK
 *
 * \details This function will uninstantiate a cryptographic context of BK which was earlier instantiated by bk_enroll() or
 * bk_start(). Once a cryptographic context is uninstantiated, BK's cryptographic functionality becomes unavailable.
 * Moreover, bk_stop() also ensures that all internal secrets related to the cryptographic context are effectively deleted
 * (zeroized), which can be used as an additional security measure against attacks.
 *
 * \pre A call to this function is only allowed when BK is in the *Initialized* state, the *Enrolled* state, the *Started*
 * state, or the *Stopped* state. Called from any other operational state, this function will return immediately with return
 * code #IID_NOT_ALLOWED, without taking any action.
 *
 * \note
 * - Calling this function from the *Stopped* state is allowed but will have no effect (BK remains in the *Stopped* state).
 * - Calling this function from the *Initialized* state is allowed, and will have as effect that bk_enroll() becomes
 *   unavailable until the next device repower or reset.
 * - Calling this function from the *Uninitialized* state is not allowed.
 *
 * \post A successful call to this function (return code equals #IID_SUCCESS) changes the operational state of BK to the *Stopped* state.
 *
 * \returns #IID_SUCCESS if successfull, or #IID_NOT_ALLOWED otherwise
 */
iid_return_t bk_stop(void);

/**
 @}
  */

/**
 * \name BK Randomness and Device-Key Generation Functions
 * \anchor bk_funcgroup_safe
@{
 */

/**
 * \brief (Re)generates a device-unique symmetric key
 *
 * \details This function will (re)generate a device-unique symmetric key for the cryptographic context which was earlier
 * instantiated by bk_enroll() or re-instantiated by bk_start(). The length of the generated key depends on the specified
 * key type. For each key type, bk_get_key() can (re)generate up to 256 independent device key values, controlled by the key
 * index input parameter. A call to bk_get_key() with the same input parameter values (\p key_type and \p index) on the same
 * device instantiated with the same cryptographic context, will always return the same key value.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in]  key_type The type of the device key that will be generated. This must be a value of the enumeration type
 *                      ::bk_sym_key_type_t.
 * \param[in]  index    An integer value in the range [0:255] indicating the index of the device key that will be generated
 *                      for the specified key type. For each index value, a key is generated which is completely independent
 *                      from keys generated by other key index values.
 * \param[out] key      Pointer to an output buffer which will hold the generated device key.\n \b Note: the size of the key
 *                      buffer must be large enough to hold a key type as specified by the \p key_type input parameter, as
 *                      indicated [here](\ref bk_sym_key_type).
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS or #IID_NOT_ALLOWED otherwise
 */
iid_return_t bk_get_key(
    const bk_sym_key_type_t         key_type,
    const uint8_t                   index,
          uint8_t           * const key);

/**
 * \brief Generates a sequence of random bytes
 *
 * \details This function will generate a sequence of random bytes using a cryptographically secure random number generator
 * which is seeded with unpredictable noise entropy from the device.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in]  number_of_bytes Positive integer in the range [1:65535] which specifies the number of random bytes that will
 *                             be returned. The size, in bytes, of the allocated output buffer pointed to by data_buffer
 *                             needs to be at least equal to this value.
 * \param[out] data_buffer     Pointer to an output buffer of byte size \p number_of_bytes which will hold the requested
 #                             random bytes.
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS or #IID_NOT_ALLOWED otherwise
 */
iid_return_t bk_generate_random(
    const uint16_t         number_of_bytes,
          uint8_t  * const data_buffer);

/**
 * \brief Generates an elliptic-curve private key
 *
 * \details This function will generate a random or a device-unique elliptic curve private key for the cryptographic context
 * which was earlier instantiated by bk_enroll() or re-instantiated by bk_start(). The length of the generated private key
 * depends on the specified [curve](\ref bk_ecc_curve). For each curve option, bk_get_private_key() can (re)generate
 * multiple device-unique key values by altering the usage context input parameter. The usage context input can be used for
 * key diversification in the application, and/or to include application-provided key information or entropy into the key
 * generation process.
 *
 * bk_get_private_key() can generate elliptic curve private keys from two possible sources:
 * - device-unique private keys derived from the device's secret fingerprint. In this case, providing the same input
 *   parameter values (curve and usage context) on the same device instantiated with the same cryptographic context, will
 *   always return the same private key value.
 * - randomly generated private keys derived from the device's power-up noise. In this case, always a fresh and
 *   unpredictably random private key value is returned.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in]  curve                Specifies the named elliptic curve on which the considered private key is defined. It
 *                                  must be a valid curve type of the ::bk_ecc_curve_t enumeration.
 * \param[in]  usage_context        Pointer to an input buffer of byte size \p usage_context_length which holds the
 *                                  (optional) usage context. When used, the entropy of this buffer is included in the
 *                                  private key derivation for private keys derived from the device fingerprint
 *                                  (\p key_source = ::BK_ECC_KEY_SOURCE_PUF_DERIVED) or from the device's random number
 *                                  generator (\p key_source = ::BK_ECC_KEY_SOURCE_RANDOM).\n \b Note: providing a usage
 *                                  context is optional. If the specified \p usage_context_length is 0, usage context is
 *                                  taken into account and \p usage_context must be NULL.
 * \param[in]  usage_context_length The size in bytes of the \p usage_context buffer. If this size is set to 0, no usage
 *                                  context is taken into account and \p usage_context must be NULL.
 * \param[in]  key_source           Specifies the source of the elliptic curve private key. It must be a valid source of the
 *                                  ::bk_key_source_id enumeration. The allowed private key sources, and their meaning, are
 *                                  explained [here](\ref bk_defgroup_ecckeysource). The allowed key sources for
 *                                  bk_get_private_key() are:
 *                                  - ::BK_ECC_KEY_SOURCE_PUF_DERIVED: the private key is derived from the device
 *                                    fingerprint and (optionally) the provided usage context.
 *                                  - ::BK_ECC_KEY_SOURCE_RANDOM: the private key is uniformly randomly generated from BK's
 *                                    internal random number generator and (optionally) the provided usage context
 * \param[out] private_key          Pointer to an output buffer which will hold the generated elliptic curve private key.
 *                                  The private key value is provided in raw binary format, in network byte order
 *                                  representation.\n \b Note: the size of the private key buffer must be large enough to
 *                                  hold a private key as specified by the [curve](\ref bk_ecc_curve) parameter. For the
 *                                  given configuration, the buffer type ::bk_ecc_private_key_t can hold all raw private
 *                                  keys.
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS or #IID_NOT_ALLOWED otherwise
 */
iid_return_t bk_get_private_key(
    const bk_ecc_curve_t              curve,
    const uint8_t             * const usage_context,
    const uint32_t                    usage_context_length,
    const bk_ecc_key_source_t         key_source,
          uint8_t             * const private_key);

/**
 @}
  */

/**
 * \name BK Symmetric Key Crypto Functions
 * \anchor bk_funcgroup_plus
@{
 */

/**
 * \brief Securely wraps a presented key into a device-unique key code
 *
 * \details This function will securely wrap (authenticated encrypt) an externally provided application key into a key code.
 * The length of the provided key must be a multiple of 4 bytes, with a minimum of 4 bytes and maximum of 1024 bytes. The
 * length of the generated key code will be the length of the provided key incremented with the constant size of the key
 * code header (#BK_USER_KEY_CODE_NONKEY_BYTES). In addition to a variable-length key, the application can provide an index value
 * which gets wrapped alongside the key. The application can assign a custom meaning to this index which relates to the
 * context of the key. When the key code is unwrapped again with bk_unwrap(), the index will also be returned.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in]  index      An integer value in the range [0:255] indicating the index of the key-to-be-wrapped. For bk_wrap()
 *                        and bk_unwrap(), the index is an application-defined value that gets wrapped (in bk_wrap()) and
 *                        unwrapped (in bk_unwrap()) alongside the actual key value. The application using BK can use it,
 *                        e.g. to specify context-information associated to the key.
 * \param[in]  key        Pointer to an input buffer with a size of \p key_length bytes, which holds the plain
 *                        key-to-be-wrapped.\n \b Note: the address must be at a 32-bit boundary, so the lowest two bits of
 *                        the address must be 0.
 * \param[in]  key_length The length, in bytes, of the key-to-be-wrapped. This must be an integer value in the range
 *                        [4:4:1024], i.e. the smallest allowed value is 4, the largest is 1024, and only values that are a
 *                        multiple of 4 are allowed.
 * \param[out] key_code   Pointer to an output buffer of size (\p key_length + #BK_USER_KEY_CODE_NONKEY_BYTES) bytes which
 *                        will hold the generated key code containing the wrapped key.\n \b Note: the address must be at a
 *                        32-bit boundary, so the lowest two bits of the address must be 0.
 *
 * \returns #IID_SUCCESS if succesfull, or #IID_INVALID_PARAMETERS or #IID_NOT_ALLOWED otherwise
 */
iid_return_t bk_wrap(
    const uint8_t          index,
    const uint8_t  * const key,
    const uint16_t         key_length,
          uint8_t  * const key_code);

/**
 * \brief Unwraps the key from a device-unique key code
 *
 * \details This function will successfully unwrap (decrypt and authenticate) a provided key code, given that it is called
 * on the same device and in the same cryptographic context that was used to produce the key code with bk_wrap(). In
 * addition to the originally wrapped key, bk_unwrap() will also return the index value that got wrapped alongside the key.
 * The application can parse this index value to determine the context of the key.
 *
 * bk_unwrap() can determine the exact length of the key code automatically by parsing the key code header, so the
 * application does not need to provide the key code length. However, the calling software does need to assure that the
 * allocated key code buffer is large enough to hold the complete key code string.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in]  key_code   Pointer to an input buffer which holds the key-code-to-be-unwrapped.\n \b Note: the address must
 *                        be at a 32-bit boundary, so the lowest two bits of the address must be 0.
 * \param[out] key        Pointer to an output buffer with a size of (sizeof(\p key_code) - #BK_USER_KEY_CODE_NONKEY_BYTES)
 *                        bytes that will hold the unwrapped key.
 * \param[out] key_length Pointer to an output buffer with a size of 2 bytes (representing a uint16_t) which will hold the
 *                        size in bytes, of the unwrapped key.
 * \param[out] index      Pointer to an output buffer with a size of 1 byte which will hold the index value which was
 *                        wrapped alongside the unwrapped key. For bk_wrap() and bk_unwrap(), the index is an
 *                        application-defined value that gets wrapped (in bk_wrap()) and unwrapped (in bk_unwrap())
 *                        alongside the actual key value. The application using BK can use it, e.g. to specify
 *                        context-information associated to the key.
 *
 * \returns #IID_SUCCESS if succesfull, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED or #IID_INVALID_KEY_CODE otherwise
 */
iid_return_t bk_unwrap(
    const uint8_t  * const key_code,
          uint8_t  * const key,
          uint16_t * const key_length,
          uint8_t  * const index);

/**
 @}
  */

/**
 * \name BK Elliptic Curve Crypto Functions
 * \anchor bk_funcgroup_pro
@{
 */

/**
 * \brief Derives an elliptic curve public key from a private key.
 *
 * \details This function derives the elliptic curve public key corresponding to a private key created with
 * bk_get_private_key(), and outputs the public key.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] use_point_compression \b Note: this flag is present for future use compatibility, but is not used for this
 *                                  product version of BK. For this product version, this value has to be set to False (no
 *                                  point compression). Any other values will result in the return code
 *                                  #IID_INVALID_PARAMETERS.
 * \param[in] curve                 Specifies the named elliptic curve on which the private key is defined. It must be a
 *                                  valid curve type of the ::bk_ecc_curve_t enumeration.
 * \param[in] private_key           Pointer to an input buffer that holds the elliptic curve private key. The expected input
 *                                  is binary in network byte order representation. Its size in bytes is determined by the
 *                                  used [curve](\ref bk_ecc_curve). For the given configuration, the buffer type
 *                                  ::bk_ecc_private_key_t can hold all raw private keys.\n \b Note: the address must be at
 *                                  a 32-bit boundary, so the lowest two bits of the address must be 0.
 * \param[out] std_public_key       Pointer to an output buffer that will hold the elliptic curve public key computed from
 *                                  the \p private_key input. The output is in X9.62 binary format. Its size in bytes is
 *                                  determined by the used [curve](\ref bk_ecc_curve). For the given configuration, the
 *                                  buffer type ::bk_ecc_std_public_key_t can hold all X9.62 binary format public keys.\n \b
 *                                  Note: the address must be at a 32-bit boundary, so the lowest two bits of the address
 *                                  must be 0.
 *
 * \returns #IID_SUCCESS or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED or #IID_INVALID_PRIVATE_KEY
 */
iid_return_t bk_derive_public_key(
    const bool                   use_point_compression,
    const bk_ecc_curve_t         curve,
    const uint8_t        * const private_key,
          uint8_t        * const std_public_key);

/**
 * \brief Protects an elliptic curve private key into a private key code, ready for use with BK's elliptic curve functions.
 *
 * \details This function transforms an elliptic curve private key into a protected private key code which is only usable
 * within the same cryptographic context, and on the same unique device, it was created on. This function can take private
 * keys from three possible sources:
 * - private keys derived from the device's secret fingerprint
 * - randomly generated private keys
 * - user-provided private keys
 *
 * Alongside the private key values, this function also stores the curve and key purpose flags in the private key code
 * format. This makes the future use of a generated private key code self-contained, i.e. a consuming function knows on
 * which curve the contained private key is defined, and for which purposes it is allowed to be used.
 *
 * \note The protection mechanisms for transforming private keys into private key codes are similar as for the bk_wrap()
 * function, but private key codes cannot be unwrapped by bk_unwrap(). The underlying internal keys used by
 * bk_create_private_key() for protecting private key codes are also different as for key codes generated by bk_wrap(). Once
 * packed into a private key code, the actual private key values can no longer be publicly retrieved by BK.
 *
 * \note The generation mechanisms for creating device-unique and random private keys are similar as for
 * bk_get_private_key(), but private keys generated by bk_get_private_key() and bk_create_private_key() are strongly
 * cryptographically separated. This entails that calling bk_get_private_key() and bk_create_private_key() with equal
 * parameters will always result in completely different private key values.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] curve                Specifies the named elliptic curve on which the considered private key is defined. It
 *                                 must be a valid curve type of the ::bk_ecc_curve_t enumeration.
 * \param[in] purpose_flags        Flag which specifies the [purpose flags](\ref bk_defgroup_keypurpose) of the private key.
 * \param[in] usage_context        Pointer to an input buffer of size \p usage_context_length bytes which holds the
 *                                 (optional) usage context. When used, the entropy of this buffer is included in the
 *                                 private key derivation for private keys derived from the device fingerprint
 *                                 (\p key_source = ::BK_ECC_KEY_SOURCE_PUF_DERIVED) or from the device's random number
 *                                 generator (\p key_source = ::BK_ECC_KEY_SOURCE_RANDOM).\n \b Note: providing a usage
 *                                 context is optional. If the specified \p usage_context_length is 0, no usage context is
 *                                 taken into account and \p usage_context must be NULL.
 * \param[in] usage_context_length The size in bytes of the \p usage_context buffer. If this length is set to 0, no usage
 *                                 context is taken into account and \p usage_context must be NULL.
 * \param[in] key_source           Specifies the source of the elliptic curve private key. It must be a valid source of the
 *                                 ::bk_ecc_key_source_t enumeration. Possible key sources and their meaning, are explained
 *                                 [here](\ref bk_key_source_id). The allowed key sources for bk_create_private_key() are:
 *                                  - ::BK_ECC_KEY_SOURCE_PUF_DERIVED: the private key is derived from the device
 *                                    fingerprint and (optionally) the provided usage context.
 *                                  - ::BK_ECC_KEY_SOURCE_RANDOM: the private key is uniformly randomly generated from BK's
 *                                    internal random number generator and (optionally) the provided usage context
 *                                  - ::BK_ECC_KEY_SOURCE_USER_PROVIDED: the private key is provided externally. When this
 *                                    key source is selected, \p usage_context is not used and \p private_key is used
 *                                    directly with only a check that it is a well-formed private key for the specified
 *                                    curve.
 *                                 The resulting private key will be wrapped by a device-unique PUF key, hence the resulting
 *                                 private key code can only be used within the same cryptographic context on the same
 *                                 device.\n
 *                                 \b Note: for ::BK_ECC_KEY_SOURCE_PUF_DERIVED and ::BK_ECC_KEY_SOURCE_RANDOM, the
 *                                 \p usage_context entropy, if present, is added to the key derivation process to provide
 *                                 a secure fallback or key diversification.
 * \param[in] private_key          Pointer to an input buffer that holds the private key used when \p key_source is
 *                                 ::BK_ECC_KEY_SOURCE_USER_PROVIDED. The expected input is binary in network byte order
 *                                 representation. Its size in bytes is determined by the used [curve](\ref bk_ecc_curve).
 *                                 For the given configuration, the buffer type ::bk_ecc_private_key_t can hold all raw
 *                                 private keys.\n \b Note: the address must be at a 32-bit boundary, so the lowest two bits
 *                                 of the address must be 0.\n \b Note: for key sources other than
 *                                 ::BK_ECC_KEY_SOURCE_USER_PROVIDED, this input is not used.
 * \param[out] private_key_code    Pointer to an output buffer for a ::bk_ecc_private_key_code_t which will hold the created
 *                                 elliptic curve private key code. \n \b Note: the address must be at a 32-bit boundary, so
 *                                 the lowest two bits of the address must be 0.
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED or #IID_INVALID_PRIVATE_KEY otherwise
 */
iid_return_t bk_create_private_key(
    const bk_ecc_curve_t                    curve,
    const bk_ecc_key_purpose_t              purpose_flags,
    const uint8_t                   * const usage_context,
    const uint32_t                          usage_context_length,
    const bk_ecc_key_source_t               key_source,
    const uint8_t                   * const private_key,
          bk_ecc_private_key_code_t * const private_key_code);


/**
 * \brief Computes an elliptic curve public key code from a private key code, to be used with BK's elliptic curve functions.
 *
 * \details This function computes the elliptic curve public key corresponding to a private key code created with
 * bk_create_private_key(), and outputs the public key in a corresponding public key code format. The curve and purpose
 * flags of the public key (code) will be the same as the one of the provided private key (code).
 *
 * \note The protection mechanisms for storing public keys as public key codes are similar as for the bk_wrap() function,
 * but public key codes cannot be unwrapped by bk_unwrap(). The underlying internal keys used by
 * bk_compute_public_from_private_key() for protecting public key codes are also different as for key codes generated by
 * bk_wrap(). If needed, the function bk_export_public_key() can be used to retrieve the public key value contained in a
 * public key code.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] private_key_code Pointer to an input buffer for a ::bk_ecc_private_key_code_t which contains an elliptic curve
 *                             private key code created by bk_create_private_key().\n \b Note: the address must be at a
 *                             32-bit boundary, so the lowest two bits of the address must be 0.
 * \param[out] public_key_code Pointer to an output buffer for a ::bk_ecc_public_key_code_t which will hold the created
 *                             elliptic curve public key code. \n \b Note: the address must be at a 32-bit boundary, so the
 *                             lowest two bits of the address must be 0.
 *
 * \returns #IID_SUCCESS if succesfful, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, #IID_INVALID_PRIVATE_KEY_CODE otherwise
 */
iid_return_t bk_compute_public_from_private_key(
    const bk_ecc_private_key_code_t * const private_key_code,
          bk_ecc_public_key_code_t  * const public_key_code);

/**
 * \brief Imports an elliptic curve public key to the internal protected public key code format, ready for use with BK's
 * elliptic curve functions.
 *
 * \details This function imports an elliptic curve public key from a provided X9.62 binary format (compressed or
 * uncompressed) to a corresponding public key code format. The curve and purpose flags of the public key (code) are also
 * provided as inputs and stored in the public key code.
 *
 * \note The protection mechanisms for storing public keys as public key codes are similar as for the bk_wrap() function,
 * but public key codes cannot be unwrapped by bk_unwrap(). The underlying internal keys used by bk_import_public_key() for
 * protecting public key codes are also different as for key codes generated by bk_wrap(). If needed, the function
 * bk_export_public_key() can be used to retrieve the public key value contained in a public key code.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] curve            Specifies the named elliptic curve on which the public key is defined. It must be a valid
 *                             curve type of the ::bk_ecc_curve_t enumeration.
 * \param[in] purpose_flags    Flag which specifies the [purpose flags](\ref bk_defgroup_keypurpose) of the public key.
 * \param[in] std_public_key   Pointer to an input buffer which holds the elliptic curve public key to be imported. The
 *                             expected input is in X9.62 binary format. Its size in bytes is determined by the used [curve]
 *                             (\ref bk_ecc_curve). For the given configuration, the buffer type ::bk_ecc_std_public_key_t
 *                             can hold all X9.62 binary format public keys.
 * \param[out] public_key_code Pointer to an output buffer for a ::bk_ecc_public_key_code_t which will hold the created
 *                             elliptic curve public key code. \n \b Note: the address must be at a 32-bit boundary, so the
 *                             lowest two bits of the address must be 0.
 *
 * \returns #IID_SUCCESS or #IID_INVALID_PARAMETERS or #IID_NOT_ALLOWED or #IID_INVALID_PUBLIC_KEY
 */
iid_return_t bk_import_public_key(
    const bk_ecc_curve_t                   curve,
    const bk_ecc_key_purpose_t             purpose_flags,
    const uint8_t                  * const std_public_key,
          bk_ecc_public_key_code_t * const public_key_code);

/**
 * \brief Exports a binary elliptic curve public key from BK's internal protected public key code format.
 *
 * \details This function exports a public key from BK's public key code format to an X9.62 binary elliptic curve public key
 * format. The curve on which the public key is defined, as well as the purpose flags stored alongside the key in public key
 * code, are returned as well.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] use_point_compression This flag is present for future use compatibility, but is not used for this product
 *                                  version of BK. For this product version, this value has to be set to False (no point
 *                                  compression). Any other values will result in the return code #IID_INVALID_PARAMETERS.
 * \param[in] public_key_code       Pointer to an input buffer for a ::bk_ecc_public_key_code_t which holds the elliptic
 *                                  curve public key code to be exported. \n \b Note: the address must be at a 32-bit
 *                                  boundary, so the lowest two bits of the address must be 0.
 * \param[out] std_public_key       Pointer to an output buffer which will hold the exported elliptic curve public key. The
 *                                  output is in X9.62 binary format. Its size in bytes is determined by the used [curve]
 *                                  (\ref bk_ecc_curve). For the given configuration, the buffer type
 *                                  ::bk_ecc_std_public_key_t can hold all X9.62 binary format public keys.
 * \param[out] curve                Pointer to an output buffer for a ::bk_ecc_curve_t which will hold the curve on which
 *                                  the exported public key is defined.
 * \param[out] purpose_flags        Pointer to an output buffer for a ::bk_ecc_key_purpose_t which will hold the
 *                                  [purpose flags](\ref bk_defgroup_keypurpose) of the exported public key.
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, #IID_INVALID_PUBLIC_KEY_CODE otherwise
 */
iid_return_t bk_export_public_key(
    const bool                             use_point_compression,
    const bk_ecc_public_key_code_t * const public_key_code,
          uint8_t                  * const std_public_key,
          bk_ecc_curve_t           * const curve,
          bk_ecc_key_purpose_t     * const purpose_flags);

/**
 * \brief ECDSA-sign signs a message, using a BK protected private key code.
 *
 * \details This function signs a message or a hash of message using ECDSA with an elliptic curve private key in the
 * internal private key code format. Signing can be done with either a random seed or a deterministically derived seed as
 * indicated by the calling application.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] private_key_code        Pointer to an input buffer for a ::bk_ecc_private_key_code_t which holds the elliptic
 *                                    curve private key code to be used for signing.\n \b Note: a private key code used for
 *                                    signing shall have been created with bk_create_private_key(), with [purpose flags]
 *                                    (\ref bk_defgroup_keypurpose) allowing its use for ECDSA operations.\n \b Note: the
 *                                    address must be at a 32-bit boundary, so the lowest two bits of the address must be 0.
 * \param[in] deterministic_signature Flag which specifies if either deterministic or non-deterministic signing will be
 *                                    used. If this value equals False, message will be signed using the standard ECDSA
 *                                    non-deterministic algorithm. Otherwise, the message will be signed using
 *                                    [a deterministic variant of ECDSA](https://tools.ietf.org/html/rfc6979).
 * \param[in] message                 Pointer to an input buffer of size \p message_length bytes which holds the message or
 *                                    the message hash that will be signed. If \p message_length is zero, \p message must be
 *                                    NULL.
 * \param[in] message_length          Value which specifies the size in bytes of the \p message buffer. If
 *                                    \p message_is_hash equals True (i.e. the provided message is actually a message hash),
 *                                    this size must be equal to the size in bytes of the used private key, as determined by
 *                                    the used [curve](\ref bk_ecc_curve). Otherwise (i.e. the provided message is a raw
 *                                    message buffer), the size in bytes must be equal to the raw message size. In this
 *                                    case, \p message_length could also be zero in which case an empty message will be
 *                                    signed. If \p message_length is zero, \p message must be NULL.
 * \param[in] message_is_hash         Flag which specifies if the provided \p message buffer contains an already hashed
 *                                    message, or a raw message buffer. If this value equals False, bk_ecdsa_sign() will
 *                                    treat the \p message buffer as a raw message, and will hash it first using SHA-256 and
 *                                    the trailing bytes will be truncated to equal the length of the used elliptic curve
 *                                    private key before signing the resulting hash. Otherwise, bk_ecdsa_sign() will treat
 *                                    the message buffer as an already hashed message, and it will be signed directly. For
 *                                    this hashed message, SHA-256, SHA-384 and SHA-512 are accepted.
 * \param[out] signature              Pointer to an output buffer which will hold the computed ECDSA signature. The
 *                                    signature is formatted as the direct concatenation of ECDSA's (r, s) values as two
 *                                    fixed-length unsigned integers, and the signature size in bytes depends on the used
 *                                    [curve](\ref bk_ecc_curve). For the given configuration, the buffer type
 *                                    ::bk_ecc_signature_t can hold all signatures. The used curve is set during the
 *                                    creation of the private key code with bk_create_private_key().
 * \param[in,out] signature_length    Pointer to an input/output buffer for a uint16 which as an input holds the allocated
 *                                    size in bytes of the \p signature buffer, and as an output will hold the exact size of
 *                                    the returned signature value. If the size provided by the input is smaller then the
 *                                    required size as determined by the used [curve](\ref bk_ecc_curve), the function
 *                                    returns with #IID_INVALID_PARAMETERS but still sets the output value at this pointer
 *                                    to the required buffer size.
 *
 * \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, #IID_INVALID_PRIVATE_KEY_CODE or
 * #IID_ECC_NOT_ALLOWED otherwise
 */
iid_return_t bk_ecdsa_sign(
    const bk_ecc_private_key_code_t * const private_key_code,
    const bool                              deterministic_signature,
    const uint8_t                   * const message,
    const uint32_t                          message_length,
    const bool                              message_is_hash,
          uint8_t                   * const signature,
          uint16_t                  * const signature_length);

/**
 * \brief Verifies an ECDSA-signed message, using a BK protected public key code.
 *
 * \details This function verifies the ECDSA signature of a message or a hash of message with an elliptic curve public key
 * in the internal public key code format.
 *
 * \note This function has no explicit output parameter(s), but the result of this function will be contained in its return
 * code. Upon return of #IID_SUCCESS, the presented \p signature was valid for the \p message, while upon return of
 * #IID_INVALID_SIGNATURE, the presented \p signature was not valid for the \p message.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] public_key_code  Pointer to an input buffer for a ::bk_ecc_public_key_code_t which holds the elliptic curve
 *                             public key code to be used for verification.\n \b Note: a public key code used for signature
 *                             verification shall have been computed with bk_compute_public_from_private_key() or imported
 *                             with bk_import_public_key(), with [purpose flags](\ref bk_defgroup_keypurpose) allowing its
 *                             use for ECDSA operations.\n \b Note: the address must be at a 32-bit boundary, so the lowest
 *                             two bits of the address must be 0.
 * \param[in] message          Pointer to an input buffer of size \p message_length bytes which holds the message or the
 *                             message hash for which the signature will be verified. If \p message_length is zero,
 *                             \p message must be NULL.
 * \param[in] message_length   Value which specifies the size in bytes of the \p message buffer. If \p message_is_hash
 *                             equals True (i.e. the provided message is actually a message hash), this length must be equal
 *                             to the size in bytes of the used private key, as determined by the used [curve]
 *                             (\ref bk_ecc_curve). Otherwise (i.e. the provided message is a raw message buffer), the size
 *                             in bytes must be equal to the raw message length. In this case, \p message_length could also
 *                             be zero in which case an empty message will be verified. If \p message_length is zero,
 *                             \p message must be NULL.
 * \param[in] message_is_hash  Flag which specifies if the provided \p message buffer contains an already hashed message, or
 *                             a raw message buffer. If this value equals False, bk_ecdsa_verify() will treat the \p message
 *                             buffer as a raw message, and will be hash it first using SHA-256 and the trailing bytes will
 *                             be truncated to equal the length of the used elliptic curve private key before verifying the
 *                             resulting hash. Otherwise, bk_ecdsa_verify() will treat the message buffer as an already
 *                             hashed message, and it will be verified directly. For this hashed message, SHA-256,
 *                             SHA-384 and SHA-512 are accepted.
 * \param[in] signature        Pointer to an input buffer of size \p signature_length bytes, which holds the ECDSA signature
 *                             to be verified. The expected format of the provided signature is the same format as generated
 *                             by bk_ecdsa_sign().
 * \param[in] signature_length The size in bytes of the signature to be verified, which depends on the [curve]
 *                             (\ref bk_ecc_curve) used for the signing operation.
 *
 * \returns #IID_SUCCESS or #IID_INVALID_SIGNATURE if successfull, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED,
 * #IID_INVALID_PUBLIC_KEY_CODE or #IID_ECC_NOT_ALLOWED otherwise
 */
iid_return_t bk_ecdsa_verify(
    const bk_ecc_public_key_code_t * const public_key_code,
    const uint8_t                  * const message,
    const uint32_t                         message_length,
    const bool                             message_is_hash,
    const uint8_t                  * const signature,
    const uint16_t                         signature_length);

/**
 * \brief Computes an ECDH shared secret, from a pair of BK protected public and private key codes.
 *
 * \details This function computes a shared secret value using the ECDH algorithm on the provided private and public key
 * (codes). The returned shared secret comprises the X-coordinate of the mutual curve point computed with the elliptic curve
 * Diffie-Hellman method.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] private_key_code Pointer to an input buffer for a ::bk_ecc_private_key_code_t which holds the elliptic curve
 *                             private key code to be used for the Diffie-Hellmann operation.\n \b Note: a private key code
 *                             used for shared secret computation shall have been created with bk_create_private_key(), with
 *                             [purpose flags](\ref bk_defgroup_keypurpose) allowing its use for ECDH operations.\n \b Note:
 *                             the address must be at a 32-bit boundary, so the lowest two bits of the address must be 0.
 * \param[in] public_key_code  Pointer to an input buffer for a ::bk_ecc_public_key_code_t which holds the elliptic curve
 *                             public key code to be used for the Diffie-Hellmann operation.\n \b Note: a public key code
 *                             used for shared secret computation shall have been computed with
 *                             bk_compute_public_from_private_key() or imported with bk_import_public_key(), with
 *                             [purpose flags](\ref bk_defgroup_keypurpose) allowing its use for ECDH operations.\n \b Note:
 *                             a public key code used for shared secret computation shall contain a public key defined over
 *                             the same curve as the simultaneously provided private key code.\n \b Note: the address must
 *                             be at a 32-bit boundary, so the lowest two bits of the address must be 0.
 * \param[out] shared_secret   Pointer to an output buffer which will hold the computed shared secret. The size in bytes of
 *                             the shared secret depends on the used [curve](\ref bk_ecc_curve). For the given
 *                             configuration, the buffer type ::bk_ecc_shared_secret_t can hold all shared secrets. The
 *                             shared secret will be equal to the X-coordinate of the commonly derived point on the elliptic
 *                             curve.
 *
 * \returns #IID_SUCCESS if successful, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, #IID_INVALID_PRIVATE_KEY_CODE,
 * #IID_INVALID_PUBLIC_KEY_CODE, #IID_ECC_NOT_ALLOWED or #IID_CURVE_MISMATCH otherwise
 */
iid_return_t bk_ecdh_shared_secret(
    const bk_ecc_private_key_code_t * const private_key_code,
    const bk_ecc_public_key_code_t  * const public_key_code,
          uint8_t                   * const shared_secret);

/**
 * \brief Generates a BK elliptic-curve cryptogram, providing message encryption and authentication.
 *
 * \details This function packs a provided plaintext into a BK-specific protected cryptogram format, based on an elliptic
 * curve hybrid encryption scheme. The cryptogram format offers protection for confidentiality, integrity, sender
 * authentication and replay. A cryptogram is the single message in a one-pass protocol from a sender to a receiver. The
 * cryptogram generation is based simultaneously on the sender's private key and the receiver's public key. Both private and
 * public key are provided as key codes which have to be defined over the same elliptic curve, and both keycodes must have
 * their purpose flags set to allow the keys being used for ECDH/encryption.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 *  \param[in] receiver_public_key_code Pointer to an input buffer for a ::bk_ecc_public_key_code_t which holds the elliptic
 *                                      curve public key code of the receiver to whom the generated cryptogram will be sent.
 *                                      \n \b Note: a public key code used for cryptogram generation shall have been
 *                                      computed with bk_compute_public_from_private_key() or imported with
 *                                      bk_import_public_key(), with [purpose flags](\ref bk_defgroup_keypurpose) allowing
 *                                      its use for ECDH/encryption operations.\n \b Note: in order to have secure receiver
 *                                      authentication (i.e. assurance to the sender that only the intended receiver will be
 *                                      able to unpack the cryptogram), the public key contained in
 *                                      \p receiver_public_key_code shall have been verified in an independent manner (e.g.
 *                                      through certificate validation), or it shall come from an independent trusted source.
 *                                      \n \b Note: the address must be at a 32-bit boundary, so the lowest two bits of the
 *                                      address must be 0.
 *  \param[in] sender_private_key_code  Pointer to an input buffer for a ::bk_ecc_private_key_code_t which holds the
 *                                      elliptic curve private key code of the sender whom will send the generated
 *                                      cryptogram.\n \b Note: a private key code used for cryptogram generation shall have
 *                                      been created with bk_create_private_key(), with
 *                                      [purpose flags](\ref bk_defgroup_keypurpose) allowing its use for ECDH/encryption
 *                                      operations.\n \b Note: a private key code used for cryptogram generation shall
 *                                      contain a private key defined over the same curve as the simultaneously provided
 *                                      public key code.\n \b Note: the address must be at a 32-bit boundary, so the lowest
 *                                      two bits of the address must be 0.
 *  \param[in] cryptogram_type          Value which specifies the cryptogram type to be generated. It must be a valid value
 *                                      of the ::bk_ecc_cryptogram_type_t enumeration, as listed [here]
 *                                      (\ref bk_ecc_cryptogram_type).
 *  \param[in,out] counter64            Pointer to an input/output buffer with a size of 8 bytes (interpreted as a uint64),
 *                                      which as an input holds the current 64-bit monotonic counter used for cryptogram
 *                                      replay protection, and as an output will hold the new counter value after successful
 *                                      function completion.\n \b Note: a separate counter buffer shall be used for each
 *                                      distinct sender-receiver key pair. The calling application needs to retrieve this
 *                                      buffer from persistent storage before each call to bk_generate_cryptogram(), and
 *                                      store back its updated value in persistent storage after the function completes
 *                                      successfully (return code is #IID_SUCCESS). Upon first use of a counter buffer for a
 *                                      sender-receiver pair, the counter buffer needs to be initialized to all-zero bytes,
 *                                      after the initial validation of the receiver's public key. \n \b Note: the address
 *                                      must be at a 32-bit boundary, so the lowest two bits of the address must be 0.
 *  \param[in] plaintext                Pointer to an input buffer of size \p plaintext_length bytes which holds the
 *                                      plaintext which will be packed in the cryptogram.\n \b Note: the address must be at
 *                                      a 32-bit boundary, so the lowest two bits of the address must be 0.
 *  \param[in] plaintext_length         Value which specifies the size in bytes of \p plaintext. Its value must be positive
 *                                      (> 0) and a multiple of 4.
 *  \param[out] cryptogram              Pointer to an output buffer of size \p cryptogram_length bytes which will hold the
 *                                      generated cryptogram. \n \b Note: the address must be at a 32-bit boundary, so the
 *                                      lowest two bits of the address must be 0.
 *  \param[in,out] cryptogram_length    Pointer to an input/output buffer for a uint32, which as an input holds the
 *                                      allocated size in bytes of the \p cryptogram output buffer, and as an output will
 *                                      hold the exact size in bytes of the returned cryptogram. If the size provided by the
 *                                      input is smaller then the required size as determined by the used [cryptogram type]
 *                                      (\ref bk_ecc_cryptogram_type) and [curve](\ref bk_ecc_curve), the function returns
 *                                      with #IID_INVALID_PARAMETERS but still sets the output value at this pointer to the
 *                                      required buffer size.
 *
 *  \returns #IID_SUCCESS if successfull, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, #IID_INVALID_COUNTER,
 *  #IID_INVALID_PRIVATE_KEY_CODE, #IID_INVALID_PUBLIC_KEY_CODE, #IID_ECC_NOT_ALLOWED or #IID_CURVE_MISMATCH otherwise
 */

iid_return_t bk_generate_cryptogram(
    const bk_ecc_public_key_code_t  * const receiver_public_key_code,
    const bk_ecc_private_key_code_t * const sender_private_key_code,
    const bk_ecc_cryptogram_type_t          cryptogram_type,
          uint8_t                   * const counter64,
    const uint8_t                   * const plaintext,
    const uint32_t                          plaintext_length,
          uint8_t                   * const cryptogram,
          uint32_t                  * const cryptogram_length);

/**
 * \brief Processes a received BK elliptic-curve cryptogram to retrieve the contained message.
 *
 *  \details This function processes a received cryptogram in a BK-specific protected cryptogram format, to retrieve the
 *  contained plaintext, using an elliptic curve hybrid decryption scheme. The cryptogram format offers protection for
 *  confidentiality, integrity, sender authentication and replay. A cryptogram is the single message in a one-pass protocol
 *  from a sender to a receiver. The cryptogram processing is based simultaneously on the receiver's private key and the
 *  sender's public key. Both private and public key are provided as key codes which shall have been defined over the same
 *  elliptic curve, and which shall both have their purpose flags set to allow their use for ECDH/encryption. A received
 *  cryptogram can only be correctly processed (decrypted and authenticated) if the provided receiver private key and sender
 *  public key correspond respectively to the receiver public key and sender private key used to create the cryptogram, e.g.
 *  using bk_generate_cryptogram().
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] receiver_private_key_code Pointer to an input buffer for a ::bk_ecc_private_key_code_t which holds the
 *                                      elliptic curve private key code of the receiver by whom the cryptogram is processed.
 *                                      \n \b Note: a private key code used for cryptogram processing shall have been
 *                                      created with bk_create_private_key(), with [purpose flags]
 *                                      (\ref bk_defgroup_keypurpose) allowing its use for ECDH/encryption operations.
 *                                      \n \b Note: the address must be at a 32-bit boundary, so the lowest two bits of the
 *                                      address must be 0.
 * \param[in] sender_public_key_code    Pointer to an input buffer for a ::bk_ecc_public_key_code_t which holds the elliptic
 *                                      curve public key code of the sender from whom the to-be-processed cryptogram was
 *                                      received.\n \b Note: a public key code used for cryptogram processing shall have
 *                                      been created with bk_compute_public_from_private_key() or bk_import_public_key(),
 *                                      with [purpose flags](\ref bk_defgroup_keypurpose) allowing its use for ECDH/
 *                                      encryption operations. \n \b Note: in order to have secure sender authentication
 *                                      (i.e. assurance to the receiver that the cryptogram comes from the expected sender),
 *                                      the public key contained in \p sender_public_key_code shall have been verified in an
 *                                      independent manner (e.g. through certificate validation), or it shall come from an
 *                                      independent trusted source.\n \b Note: a public key code used for cryptogram
 *                                      processing shall contain a public key defined over the same curve as the
 *                                      simultaneously provided private key code.\n \b Note: the address must be at a 32-bit
 *                                      boundary, so the lowest two bits of the address must be 0.
 * \param[out] cryptogram_type          Pointer to an output buffer for a ::bk_ecc_cryptogram_type_t which will hold the
 *                                      cryptogram type which was used to generate the cryptogram.
 * \param[in,out] counter64             Pointer to an input/output buffer with a size of 8 bytes (interpreted as a uint64),
 *                                      which as an input holds the current 64-bit monotonic counter used for cryptogram
 *                                      replay protection, and as an output will hold the new counter value after successful
 *                                      function completion.\n \b Note: a separate counter buffer shall be used for each
 *                                      distinct sender-receiver key pair. The calling application needs to retrieve this
 *                                      buffer from persistent storage before each call to bk_process_cryptogram(), and
 *                                      store back its updated value in persistent storage after the function completes
 *                                      successfully (return code is #IID_SUCCESS). Upon first use of a counter buffer for a
 *                                      sender-receiver pair, the counter buffer needs to be initialized to all-zero bytes,
 *                                      after the initial validation of the sender's public key.\n \b Note:
 *                                      bk_process_cryptogram() will only be able to successfully process cryptograms which
 *                                      have been generated with a corresponding counter value which is strictly larger than
 *                                      the integer value provided as input by \p counter64. This prevents replay of old
 *                                      cryptograms, but also obstructs the ability to process multiple consecutive
 *                                      cryptograms out of order. \n \b Note: the address must be at a 32-bit boundary, so
 *                                      the lowest two bits of the address must be 0.
 * \param[in] cryptogram                Pointer to an input buffer of size \p cryptogram_length bytes which holds the full
 *                                      cryptogram to be processed.\n \b Note: the address must be at a 32-bit boundary, so
 *                                      the lowest two bits of the address must be 0.
 * \param[in] cryptogram_length         Value which specifies the size in bytes of the presented \p cryptogram.\n \b Note:
 *                                      this must be the exact size of the cryptogram to be processed, as specified by the
 *                                      used [cryptogram type](\ref bk_ecc_cryptogram_type) and [curve](\ref bk_ecc_curve).
 *  \param[out] plaintext               Pointer to an output buffer of size \p plaintext_length bytes, which will hold the
 *                                      decrypted plaintext.\n \b Note: the address must be at a 32-bit boundary, so the
 *                                      lowest two bits of the address must be 0.
 *  \param[in,out] plaintext_length     Pointer to an input/output buffer for a uint32, which as an input holds the
 *                                      allocated size in bytes of the \p plaintext output buffer, and as an output will
 *                                      hold the exact size in bytes of the returned plaintext. If the size provided by the
 *                                      input is smaller then the required size as determined by the used [cryptogram type]
 *                                      (\ref bk_ecc_cryptogram_type) and [curve](\ref bk_ecc_curve), the function returns
 *                                      with #IID_INVALID_PARAMETERS but still sets the output value at this pointer to the
 *                                      required buffer size.
 *
 *  \returns #IID_SUCCESS, #IID_INVALID_CRYPTOGRAM or #IID_INVALID_SENDER if successfull, or #IID_INVALID_PARAMETERS,
 *  #IID_NOT_ALLOWED, #IID_INVALID_COUNTER, #IID_INVALID_PRIVATE_KEY_CODE, #IID_INVALID_PUBLIC_KEY_CODE,
 *  #IID_ECC_NOT_ALLOWED or #IID_CURVE_MISMATCH otherwise
 */
iid_return_t bk_process_cryptogram(
    const bk_ecc_private_key_code_t * const receiver_private_key_code,
    const bk_ecc_public_key_code_t  * const sender_public_key_code,
          bk_ecc_cryptogram_type_t  * const cryptogram_type,
          uint8_t                   * const counter64,
    const uint8_t                   * const cryptogram,
    const uint32_t                          cryptogram_length,
          uint8_t                   * const plaintext,
          uint32_t                  * const plaintext_length);

/**
 * \brief Extracts the sender's public key embedded in a BK elliptic-curve cryptogram.
 *
 * \details This \a helper function extracts the sender's public key from a received BK elliptic-curve cryptogram. This
 * function is (optionally) used, prior to cryptogram processing with bk_process_cryptogram(), to facilitate the validation
 * of the sender's public key. In particular, this function is needed when then receiver upfront has no knowledge of which
 * public key was used by the sender.
 *
 * \note This is an optional helper function. Preferably, a receiver already possesses a trusted copy of the public key used
 * by the expected sender, in which case it is not necessary to use this function.
 *
 * \note This helper function solely attempts to extract the sender's public key value from a provided cryptogram. The
 * outcome of this function provides no guarantees whatsoever about the correctness/validity/authenticity of the provided
 * cryptogram or the extracted public key.
 *
 * \warning If this function is used to extract a sender's public key, it is important that the retrieved public key is
 * independently validated before calling bk_process_cryptogram() with it. This can be done, e.g. by looking up and
 * verifying the certificate corresponding to the public key, or by verifying that the public key matches a trusted copy of
 * that key, e.g. in a local database. Calling bk_process_cryptogram() with an *unvalidated* sender public key voids the
 * sender authentication property of the cryptogram functionality.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK.
 *
 * \param[in] use_point_compression This flag is present for future use compatibility, but is not used for this product
 *                                  version of BK. For this product version, this value has to be set to False (no point
 *                                  compression). Any other values will result in the return code #IID_INVALID_PARAMETERS.
 * \param[in] curve                 Specifies the named elliptic curve on which the cryptogram is defined. It must be a
 *                                  valid curve type of the ::bk_ecc_curve_t enumeration.
 * \param[in] cryptogram            Pointer to an input buffer of size \p cryptogram_length bytes, which holds the full
 *                                  cryptogram.\n \b Note: the address must be at a 32-bit boundary, so the lowest two bits
 *                                  of the address must be 0.
 * \param[in] cryptogram_length     Value which specifies the size in bytes of the presented \p cryptogram.\n \b Note: this
 *                                  must be the exact size of the cryptogram to be processed, as specified by the used
 *                                  [cryptogram type](\ref bk_ecc_cryptogram_type) and [curve](\ref bk_ecc_curve).
 * \param[out] std_public_key       Pointer to an output buffer which will hold the extracted public key in X9.62 binary
 *                                  format. The size in bytes is determined by the used [curve](\ref bk_ecc_curve). For the
 *                                  given configuration, the buffer type ::bk_ecc_std_public_key_t can hold all X9.62 binary
 *                                  format public keys.
 *
 * \returns #IID_SUCCESS or #IID_INVALID_CRYPTOGRAM if successfull, or #IID_INVALID_PARAMETERS or #IID_NOT_ALLOWED otherwise
 */
iid_return_t bk_get_public_key_from_cryptogram(
    const bool                   use_point_compression,
    const bk_ecc_curve_t         curve,
    const uint8_t        * const cryptogram,
    const uint32_t               cryptogram_length,
          uint8_t        * const std_public_key);

/**
 @}
  */

/**
 * \name BK PKI Functions
 * \anchor bk_funcgroup_pki
@{
 */

/**
 * \brief Precomputes the (maximum) byte size of a certificate signing request (CSR).
 *
 * \details This function computes the maximum possible size a DER-encoded binary
 * [PKCS#10](https://tools.ietf.org/html/rfc2986)-formatted certificate signing request (CSR) can have for a presented
 * elliptic curve private key (code) and a given set of subject information fields identifying the owner of that key.
 *
 * \note Due to a combination of the way DER-encoding works, and the non-determinism involved in the ECDSA-signature on the
 *  CSR, the precise size in bytes of a CSR is also non-deterministic and can only be exactly determined once the CSR is
 * generated with bk_create_csr(). However, based on a given set of inputs, it is possible to precompute the maximum
 * possible byte size of the CSR, which is what bk_maxsizeof_csr() does. This is the size which needs to be allocated in
 * memory for the \p csr output buffer of bk_create_csr(). The exact byte size of a generated CSR is returned by
 * bk_create_csr() as \p csr_length.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have written the maximum size in bytes. On error the output buffer content is undefined
 *
 * \param[in] private_key_code      Pointer to an input buffer for a ::bk_ecc_private_key_code_t which holds the elliptic
 *                                  curve private key for which the CSR size needs to be computed. \n \b Note: a private key
 *                                  code used for CSR creation shall have been created with bk_create_private_key(), with
 *                                  [purpose flags](\ref bk_defgroup_keypurpose) allowing its use for ECDSA operations.
 * \param[in] use_point_compression This flag is present for future use compatibility, but is not used for this product
 *                                  version of BK. For this product version, this value has to be set to False (no point
 *                                  compression). Any other values will result in the return code #IID_INVALID_PARAMETERS.
 * \param[in] csr_subjects          Pointer to an input buffer for a ::bk_certificate_subject_t structure. The elements of
 *                                  this structure contain the values which will be included in the distinguished name of
 *                                  the CSR's Subject field.
 * \param[out] maxcsr_length        Pointer to a uint16 which will hold the maximal possible size in bytes of a CSR
 *                                  generated on the provided inputs with bk_create_csr().
 *
 * \returns #IID_SUCCESS if successful,or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED or #IID_INVALID_PRIVATE_KEY_CODE
 * otherwise
 */
iid_return_t bk_maxsizeof_csr(
    const bk_ecc_private_key_code_t * const private_key_code,
    const bool                              use_point_compression,
    const bk_certificate_subject_t  * const csr_subjects,
          uint16_t                  * const maxcsr_length);

/**
 * \brief Creates a certificate signing request (CSR) for an elliptic curve key pair.
 *
 * \details This function creates a [PKCS#10](https://tools.ietf.org/html/rfc2986)-formatted certificate signing request
 * (CSR) for a presented elliptic curve private key (code) and a limited set of subject information fields identifying the
 * owner of that key. The CSR is returned as a DER-encoded binary string.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have written the CSR as a DER-encoded byte sequence to the output buffer and have returned its size in
 * bytes. On error the output buffer content is undefined
 *
 * \param[in] private_key_code      Pointer to an input buffer for a ::bk_ecc_private_key_code_t which holds the elliptic
 *                                  curve private key for which a CSR will be created. The public key corresponding to this
 *                                  private key will be included in the CSR, while the private key will be used to (self-)
 *                                  sign the CSR. \n \b Note: a private key code used for CSR generation shall have been
 *                                  created with bk_create_private_key(), with [purpose flags](\ref bk_defgroup_keypurpose)
 *                                  allowing its use for ECDSA operations.
 * \param[in] use_point_compression This flag is present for future use compatibility, but is not used for this product
 *                                  version of BK. For this product version, this value has to be set to False (no point
 *                                  compression). Any other values will result in the return code #IID_INVALID_PARAMETERS.
 * \param[in] csr_subjects          Pointer to an input buffer for a ::bk_certificate_subject_t structure. The elements of
 *                                  this structure contain the values which will be included in the distinguished name of
 *                                  the CSR's Subject field.
 * \param[out] csr                  Pointer to an output buffer of byte size \p csr_length, which will hold the DER-encoded
 *                                  binary CSR.
 * \param[in,out] csr_length        Pointer to an input/output buffer for a uint16, which as an input holds the allocated
 *                                  size in bytes of the \p csr output buffer, and as an output (upon successful execution)
 *                                  will hold the exact size in bytes of the returned CSR. The exact size of a CSR is not
 *                                  entirely deterministic from its inputs, but the maximum possible size (which needs to be
 *                                  allocated for \p csr) can be precomputed using bk_maxsizeof_csr().
 *
 * \returns #IID_SUCCESS if successful, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, #IID_INVALID_PRIVATE_KEY_CODE or
 * #IID_PKI_BUFFER_TOO_SMALL otherwise
 */
iid_return_t bk_create_csr(
    const bk_ecc_private_key_code_t * const private_key_code,
    const bool                              use_point_compression,
    const bk_certificate_subject_t  * const csr_subjects,
          uint8_t                   * const csr,
          uint16_t                  * const csr_length);


/**
 * \brief Precomputes the (maximum) byte size of a self-signed certificate (SSC).
 *
 * \details This function computes the maximum possible size a DER-encoded binary
 * [X.509](https://tools.ietf.org/html/rfc5280)-formatted self-signed certificate can have for a presented elliptic curve
 * private key (code) and a given set of certificate information fields and subject information fields identifying the owner
 * of that key.
 *
 * \note Due to a combination of the way DER-encoding works, and the non-determinism involved in the ECDSA-signature on the
 * self-signed certificate, the precise size in bytes of a self-signed certificate is also non-deterministic and can only be
 * exactly determined once the certificate is generated with bk_create_selfsigned_certificate(). However, based on a given
 * set of inputs, it is possible to precompute the maximum possible byte size of the certificate, which is what
 * bk_maxsizeof_selfsigned_certificate() does. This is the size which needs to be allocated in memory for the \p cert output
 * buffer of bk_create_selfsigned_certificate(). The exact byte size of a generated certificate is returned by
 * bk_create_selfsigned_certificate() as \p certificate_length.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have written the maximum size in bytes. On error the output buffer content is undefined
 *
 * \param[in] private_key_code       Pointer to an input buffer for a ::bk_ecc_private_key_code_t which holds the elliptic
 *                                   curve private key for which the self-signed certificate size needs to be computed.
 *                                   \n \b Note: a private key code used for creating a self-signed certificate shall have
 *                                   been created with bk_create_private_key(), with [purpose flags]
 *                                   (\ref bk_defgroup_keypurpose) allowing its use for ECDSA operations.
 * \param[in] use_point_compression  This flag is present for future use compatibility, but is not used for this product
 *                                   version of BK. For this product version, this value has to be set to False (no point
 *                                   compression). Any other values will result in the return code #IID_INVALID_PARAMETERS.
 * \param[in] serial                 Pointer to an input buffer of byte size \p serial_length, which holds a binary string
 *                                   which will (optionally) be included as the serial number of the generated certificate.
 *                                   \n \b Note: this \a certificate serial number is not to be confused with the serial
 *                                   number which can be part of the subject's distinguished name, which is passed in the
 *                                   \p subject_sn field of \p ssc_subjects. The former is used to identify the certificate
 *                                   itself, while the latter is (part of) the identity of the certificate's owner. If
 *                                   \p serial_length is zero, \p serial must be zero.
 * \param[in] serial_length          Value which specifies the size in bytes of the \p serial buffer.\n \b Note: When this
 *                                   value is zero, no serial number field will be included in the generated certificate.
 *                                   If \p message_length is zero, \p message must be zero.
 * \param[in] ssc_subjects           Pointer to an input buffer for a ::bk_certificate_subject_t structure. The elements of
 *                                   this structure contain the values which will be included in the distinguished name of
 *                                   the certificate's Subject field.\n \b Note: since this is a \a self-signed certificate,
 *                                   the same values will be included in the \a Issuer's distinguished name as well.
 * \param[out] maxcertificate_length Pointer to a uint16 which will hold the maximal possible size in bytes of a self-signed
 *                                   certificate generated on the provided inputs with bk_create_selfsigned_certificate().
 *
 * \returns #IID_SUCCESS if successful, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED or #IID_INVALID_PRIVATE_KEY_CODE
 * otherwise
 */
iid_return_t bk_maxsizeof_selfsigned_certificate(
    const bk_ecc_private_key_code_t * const private_key_code,
    const bool                              use_point_compression,
    const uint8_t                   * const serial,
    const uint16_t                          serial_length,
    const bk_certificate_subject_t  * const ssc_subjects,
          uint16_t                  * const maxcertificate_length);

/**
 * \brief Creates a self-signed certificate (SSC) for an elliptic curve key pair.
 *
 * \details This function creates an [X.509](https://tools.ietf.org/html/rfc5280)-formatted self-signed certificate for a
 * presented elliptic curve private key (code) and a limited set of certificate information fields and subject information
 * fields identifying the owner of that key. The self-signed certificate is returned as a DER-encoded binary string.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have written the SSC as a DER-encoded byte sequence to the output buffer and have returned its size in
 * bytes. On error the output buffer content is undefined
 *
 * \param[in] private_key_code       Pointer to an input buffer for a ::bk_ecc_private_key_code_t which holds the elliptic
 *                                   curve private key for which a self-signed certificate will be created. The public key
 *                                   corresponding to this private key will be included in the certificate, while the private
 *                                   key will be used to (self-)sign the certificate.\n \b Note: a private key code used for
 *                                   creating a self-signed certificate shall have been created with bk_create_private_key(),
 *                                   with [purpose flags](\ref bk_defgroup_keypurpose) allowing its use for ECDSA operations.
 * \param[in] use_point_compression  This flag is present for future use compatibility, but is not used for this product
 *                                   version of BK. For this product version, this value has to be set to False (no point
 *                                   compression). Any other values will result in the return code #IID_INVALID_PARAMETERS.
 * \param[in] serial                 Pointer to an input buffer of byte size \p serial_length, which holds a binary string
 *                                   which will (optionally) be included as the serial number of the generated certificate.
 *                                   \n \b Note: this \a certificate serial number is not to be confused with the serial
 *                                   number which can be part of the subject's distinguished name, which is passed in
 *                                   \p subject_sn field of \p ssc_subjects. The former is used to identify the certificate
 *                                   itself, while the latter is (part of) the identity of the certificate's owner. If
 *                                   \p serial_length is zero, \p serial must be zero.
 * \param[in] serial_length          Value which specifies the size in bytes of the \p serial buffer.\n \b Note: When this
 *                                   value is zero, no serial number field will be included in the generated certificate.
 *                                   If \p message_length is zero, \p message must be zero.
 * \param[in] valid_start            Pointer to an input buffer of byte size 15 which holds a \0-terminated ASCII character
 *                                   string specifying the start date of the certificate's \a Validity period. This string
 *                                   needs to be formatted as "YYYYMMDDhhmmss".\n \b Note: the presented string is
 *                                   interpreted and included in the certificate as GMT date/time.
 * \param[in] valid_end              Pointer to an input buffer of byte size 15 which holds a \0-terminated ASCII character
 *                                   string specifying the end date of the certificate's \a Validity period. This string
 *                                   needs to be formatted as "YYYYMMDDhhmmss".\n \b Note: the presented string is
 *                                   interpreted and included in the certificate as GMT date/time.
 * \param[in] ssc_subjects           Pointer to an input buffer for a ::bk_certificate_subject_t structure. The elements of
 *                                   this structure contain the values which will be included in the distinguished name of
 *                                   the certificate's Subject field.\n \b Note: since this is a \a self-signed certificate,
 *                                   the same values will be included in the \a Issuer's distinguished name as well.
 * \param[out] certificate           Pointer to an output buffer of byte size \p certificate_length, which will hold the
 *                                   DER-encoded binary self-signed certificate.
 * \param[in,out] certificate_length Pointer to an input/output buffer for a uint16, which as an input holds the allocated
 *                                   size in bytes of the \p certificate output buffer, and as an output (upon successful
 *                                   execution) will hold the exact size in bytes of the returned certificate. The exact
 *                                   size of a certificate is not entirely deterministic from its inputs, but the maximum
 *                                   possible size (which needs to be allocated for \p certificate) can be precomputed using
 *                                   bk_maxsizeof_selfsigned_certificate().
 *
 * \returns #IID_SUCCESS if successful, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, #IID_INVALID_PRIVATE_KEY_CODE or
 * #IID_PKI_BUFFER_TOO_SMALL otherwise
 */
iid_return_t bk_create_selfsigned_certificate(
    const bk_ecc_private_key_code_t * const private_key_code,
    const bool                              use_point_compression,
    const uint8_t                   * const serial,
    const uint16_t                          serial_length,
    const char                      * const valid_start,
    const char                      * const valid_end,
    const bk_certificate_subject_t  * const ssc_subjects,
          uint8_t                   * const certificate,
          uint16_t                  * const certificate_length);

/**
 * \brief Creates a DER-encoded representation of an elliptic curve private key and its associated public key following the
 * ECPrivateKey ASN.1 syntax specified in [RFC 5915](https://tools.ietf.org/html/rfc5915).
 *
 * \details This function creates a DER-encoded representation of an elliptic curve private key and its associated public
 * key following the ECPrivateKey ASN.1 syntax specified in [RFC 5915]( https://tools.ietf.org/html/rfc5915).
 * \n \b Note: This format is equal to OpenSSL output of an EC keypair generated with the option "-outform der"
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have written the private key as a DER-encoded byte sequence to the output buffer and have returned its size
 * in bytes. On error the output buffer content is undefined
 *
 * \param[in]  curve                  Specifies the named elliptic curve on which the private key is defined. It must be a
 *                                    valid curve type of the ::bk_ecc_curve_t enumeration.
 * \param[in]  private_key            Pointer to an input buffer which holds the raw binary elliptic curve private key
 *                                    (e.g. as returned by bk_get_private_key()) to be written, with a size specified by the
 *                                    [curve](\ref bk_ecc_curve) parameter. For the given configuration, the buffer type
 *                                    ::bk_ecc_private_key_t can hold all raw private keys.
 * \param[out] ECPrivateKey           Pointer to an output buffer of byte size \p ECprivatekey_length, which will hold the
 *                                    DER-encoded binary private key and its associated public key.
 * \param[in,out] ECPrivateKey_length Pointer to an input/output buffer for a uint16, which as an input holds the allocated
 *                                    size in bytes of the \p ECPrivateKey output buffer, and as an output (upon successful
 *                                    execution) will hold the exact size in bytes of the returned ECPrivateKey. The exact
 *                                    size of an ECPrivateKey is determined by the used [curve](\ref bk_ecc_curve). For the
 *                                    given configuration, the buffer type ::bk_der_private_key_info_t can hold all
 *                                    DER-encoded representations of an elliptic curve private key .
 *
 * \returns #IID_SUCCESS if successful, or  #IID_INVALID_PRIVATE_KEY, #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED or
 * #IID_PKI_BUFFER_TOO_SMALL otherwise
 */
iid_return_t bk_write_ECPrivateKey(
    const bk_ecc_curve_t         curve,
          uint8_t        * const private_key,
          uint8_t        * const ECPrivateKey,
          uint16_t       * const ECPrivateKey_length);

/**
* \brief Creates a DER-encoded representation of an elliptic curve public key following the SubjectPublicKeyInfo ASN.1
* syntax specified in [RFC 5480](https://tools.ietf.org/html/rfc5480).
*
* \details This function creates a DER-encoded representation of an elliptic curve public key following the SubjectPublicKey
* ASN.1 syntax specified in [RFC 5480]( https://tools.ietf.org/html/rfc5480) as can be used in
* [X.509](https://tools.ietf.org/html/rfc5280) certificates. \n \b Note: This format is equal to OpenSSL public key output
* with the option "-pubout -outform der".
*
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have written the public key as a DER-encoded byte sequence to the output buffer and have returned its size
 * in bytes. On error the output buffer content is undefined
 *
 * \param[in] curve                           Specifies the named elliptic curve on which the public key is defined. It must
 *                                            be a valid curve type of the ::bk_ecc_curve_t enumeration.
 * \param[in] std_public_key                  Pointer to an input buffer which holds the elliptic curve public key to be
 *                                            written. The expected input is in X9.62 binary format (e.g. as returned by
 *                                            bk_derive_public_key() or bk_export_public_key()). Its size in bytes is
 *                                            determined by the used [curve](\ref bk_ecc_curve). For the given
 *                                            configuration, the buffer type ::bk_ecc_std_public_key_t *                                            can hold all X9.62 binary format public keys.
 * \param[out] SubjectPublicKeyInfo           Pointer to an output buffer of byte size \p SubjectPublicKeyInfo_length, which
 *                                            will hold the DER-encoded binary public key.
 * \param[in,out] SubjectPublicKeyInfo_length Pointer to an input/output buffer for a uint16, which as an input holds the
 *                                            allocated size in bytes of the \p SubjectPublicKeyInfo output buffer, and as
 *                                            an output (upon successful execution) will hold the exact size in bytes of the
 *                                            returned SubjectPublicKeyInfo. The exact size of an SubjectPublicKeyInfo is
 *                                            determined by the used [curve](\ref bk_ecc_curve). For the given
 *                                            configuration, the buffer type ::bk_der_public_key_info_t can hold all
 *                                            DER-encoded representations of an elliptic curve public key.
 *
 * \returns #IID_SUCCESS if successful, or  #IID_INVALID_PUBLIC_KEY, #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED or
 * #IID_PKI_BUFFER_TOO_SMALL otherwise.
 */
iid_return_t bk_write_SubjectPublicKeyInfo(
    const bk_ecc_curve_t         curve,
          uint8_t        * const std_public_key,
          uint8_t        * const SubjectPublicKeyInfo,
          uint16_t       * const SubjectPublicKeyInfo_length);

/**
 * \brief Creates a DER-encoded representation of an ECDSA signature following the ECDSA-Sig-Value ASN.1 syntax specified in
 * [RFC 3279](https://tools.ietf.org/html/rfc3279).
 *
 * \details This function creates a DER-encoded representation of an ECDSA signature following the ECDSA-Sig-Value ASN.1
 * syntax specified in [RFC 3279]( https://tools.ietf.org/html/rfc3279) as can be used in
 * [X.509](https://tools.ietf.org/html/rfc5280) certificates. \n \b Note: This format is equal to the default OpenSSL ECDSA
 * signature output.
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have written the ECDSA signature as a DER-encoded byte sequence to the output buffer and have returned its
 * size in bytes. On error the output buffer content is undefined
 *
 * \param[in] curve                      Specifies the named elliptic curve on which the ECDSA signature was generated. It
 *                                       must be a valid curve type of the ::bk_ecc_curve_t enumeration.
 * \param[in] signature                  Pointer to an input buffer which holds the raw binary ECDSA signature (e.g. as
 *                                       returned by bk_ecdsa_sign()) to be written, with a size in bytes is determined by
 *                                       the used [curve](\ref bk_ecc_curve). For the given configuration, the buffer type
 *                                       ::bk_ecc_signature_t can hold all signatures.
 * \param[out] ECDSA_Sig_Value           Pointer to an output buffer of byte size \p ECDSA_Sig_Value_length, which will hold
 *                                       the DER-encoded binary ECDSA signature.
 * \param[in,out] ECDSA_Sig_Value_length Pointer to an input/output buffer for a uint16, which as an input holds the
 *                                       allocated size in bytes of the \p ECDSA_Sig_Value output buffer, and as an output
 *                                       (upon successful execution) will hold the exact size in bytes of the returned
 *                                       ECDSA_Sig_Value. The exact size of an ECDSA_Sig_Value depends on the used [curve]
 *                                       (\ref bk_ecc_curve) as well as the signature data. For the given configuration, the
 *                                       buffer type ::bk_der_sig_value_t can hold all DER-encoded representations of an
 *                                       elliptic curve signature.
 *
 * \returns #IID_SUCCESS if successful, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED or #IID_PKI_BUFFER_TOO_SMALL otherwise
 */
iid_return_t bk_write_ECDSA_Sig_Value(
    const bk_ecc_curve_t         curve,
    const uint8_t        * const signature,
          uint8_t        * const ECDSA_Sig_Value,
          uint16_t       * const ECDSA_Sig_Value_length);

/**
 * \brief Reads a DER-encoded representation of an elliptic curve private key and its associated public key following the
 * ECPrivateKey ASN.1 syntax specified in [RFC 5915]( https://tools.ietf.org/html/rfc5915).
 *
 * \details This function reads a DER-encoded representation of an elliptic curve private key and its associated public key
 * following the ECPrivateKey ASN.1 syntax specified in [RFC 5915](https://tools.ietf.org/html/rfc5915).
 *
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have read the private key and its associated X9.62 binary format public key from a DER-encoded byte
 * sequence to the output buffers and have returned the curve_id. On error the output buffer content is undefined. This
 * function can return an error for corrupted DER encoded input, but also if it refers to unsupported data or method
 * identifiers.
 *
 * \param[in] ECPrivateKey            Pointer to an input buffer with the ECPrivateKey byte stream, which holds the
 *                                    DER-encoded binary private key.
 * \param[out] private_key            Pointer to an output buffer which will hold the read elliptic curve private key. The
 *                                    output will be in raw format (e.g. as accepted by bk_create_private_key()).
 * \param[in,out] private_key_length  Pointer to an input/output buffer for a uint16, which as an input holds the allocated
 *                                    size in bytes of the \p private_key output buffer, and as an output (upon successful
 *                                    execution) will hold the exact size in bytes of the returned private key. The size
 *                                    of the private key is determined by the elliptic curve ID which is encoded in the DER
 *                                    input. For the given configuration, the buffer type ::bk_ecc_private_key_t can hold
 *                                    all raw private keys.
 * \param[out] curve                  Pointer to a buffer to hold the specifier of the named elliptic curve on which the
 *                                    private key is defined. It must be a valid curve type of the ::bk_ecc_curve_t
 *                                    enumeration.
 * \returns #IID_SUCCESS if successful, or  #IID_INVALID_PRIVATE_KEY, #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED,
 * IID_PKI_INVALID_DER_INPUT or #IID_PKI_BUFFER_TOO_SMALL otherwise
 */
iid_return_t bk_read_ECPrivateKey(
    const uint8_t        * const ECPrivateKey,
          uint8_t        * const private_key,
          uint16_t       *       private_key_length,
          bk_ecc_curve_t * const curve);

/**
* \brief Reads a DER-encoded representation of an elliptic curve public key following the SubjectPublicKeyInfo ASN.1 syntax
* specified in [RFC 5480](https://tools.ietf.org/html/rfc5480).
*
* \details This function reads a DER-encoded representation of an elliptic curve public key following the SubjectPublicKey
* ASN.1 syntax specified in [RFC 5480]( https://tools.ietf.org/html/rfc5480), as can be used in
* [X.509](https://tools.ietf.org/html/rfc5280) certificates.
*
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have read the public key from a DER-encoded byte sequence to the output buffer and have returned the
 * curve_id. On error the output buffer content is undefined. This function can return an error for corrupted DER encoded
 * input, but also if it refers to unsupported data or method identifiers.
 *
 * \param[in] SubjectPublicKeyInfo       Pointer to an input buffer with the SubjectPublicKeyInfo byte stream, which holds
 *                                       the DER-encoded binary public key.
 * \param[out] std_public_key            Pointer to an output buffer which will hold the read elliptic curve public key. The
 *                                       output will be in X9.62 binary format (e.g. as accepted by bk_import_public_key()).
 * \param[in,out] std_public_key_length  Pointer to an input/output buffer for a uint16, which as an input holds the
 *                                       allocated size in bytes of the \p std_public_key output buffer, and as an output
 *                                       (upon successful execution) will hold the exact size in bytes of the returned
 *                                       public key. The size of the public key is determined by the elliptic curve ID which
 *                                       is encoded in the DER input. For the given configuration, the buffer type
 *                                       ::bk_ecc_std_public_key_t can hold all X9.62 binary format public keys.
 * \param[out] curve                     Pointer to a buffer to hold the specifier of the named elliptic curve on which the
 *                                       X9.62 binary format public key is defined. It must be a valid curve type of the
 *                                       ::bk_ecc_curve_t enumeration.
 * \returns #IID_SUCCESS if successful, or  #IID_INVALID_PUBLIC_KEY, #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, IID_PKI_INVALID_DER_INPUT or #IID_PKI_BUFFER_TOO_SMALL otherwise
 */
iid_return_t bk_read_SubjectPublicKeyInfo(
    const uint8_t        * const SubjectPublicKeyInfo,
          uint8_t        * const std_public_key,
          uint16_t       *       std_public_key_length,
          bk_ecc_curve_t * const curve);

/**
* \brief Reads a DER-encoded representation of an ECDSA signature following the ECDSA-Sig-Value ASN.1 syntax specified in
* [RFC 3279](https://tools.ietf.org/html/rfc3279).
*
* \details This function reads a DER-encoded representation of an ECDSA signature following the ECDSA-Sig-Value ASN.1 syntax
* specified in [RFC 3279]( https://tools.ietf.org/html/rfc3279), as is used in [X.509](https://tools.ietf.org/html/rfc5280)
* certificates.
*
 * \pre A call to this function is only allowed when BK is in the *Enrolled* or *Started* state. Called from any other
 * operational state, this function will return immediately with return code #IID_NOT_ALLOWED, without taking any action.
 *
 * \post A call to this function will not change the operational state of BK. If successful, signalled by #IID_SUCCESS, this
 * function will have read the ECDSA signature from a DER-encoded byte sequence to the output buffer and have returned its
 * size in bytes. On error the output buffer content is undefined. This function can return an error for corrupted DER
 * encoded input, but also if it refers to unsupported data or method identifiers.
 *
 * \param[in] curve           Specifies the named elliptic curve on which the ECDSA signature was generated. It must be a
 *                            valid curve type of the ::bk_ecc_curve_t enumeration. \n \b Note: this reader function needs
 *                            to know the curve on which the signature was generated in order to correctly parse it (since
 *                            the DER signature itself does not contain the curve information). The (expected) curve can be
 *                            obtained from the public key (code) with which one intends to validate this signature.
 * \param[in] ECDSA_Sig_Value Pointer to an input buffer holding the DER-encoded binary ECDSA signature.
 * \param[out] signature      Pointer to an output buffer which will hold the ECDSA signature (e.g. as accepted by
 *                            bk_ecdsa_verify()) to be read, with a size in bytes determined by the used [curve]
 *                            (\ref bk_ecc_curve). For the given configuration, the buffer type ::bk_ecc_signature_t can hold
 *                            all signatures.
 *
 * \returns #IID_SUCCESS if successful, or #IID_INVALID_PARAMETERS, #IID_NOT_ALLOWED, IID_PKI_INVALID_DER_INPUT or
  #IID_PKI_BUFFER_TOO_SMALL otherwise
 */
iid_return_t bk_read_ECDSA_Sig_Value(
    const bk_ecc_curve_t      curve,
    const uint8_t        * const ECDSA_Sig_Value,
          uint8_t        * const signature);

/**
 @}
  */



#ifdef __cplusplus
}
#endif

#endif /* __IID_BROADKEY__H__ */
